Package: PrInCE
Title: Predicting Interactomes from Co-Elution
Version: 1.27.0
Authors@R: c(
  person("Michael", "Skinnider", email = "michael.skinnider@msl.ubc.ca",
         role = c("aut", "trl", "cre")),
  person("R. Greg", "Stacey", email = "richard.greg.stacey@msl.ubc.ca",
         role = "ctb"),
  person("Nichollas", "Scott", email = "nichollas.scott@unimelb.edu.au",
         role = "ctb"),
  person("Anders", "Kristensen", role = "ctb"),
  person("Leonard", "Foster", email = "foster@msl.ubc.ca",
         role = c("aut", "led")))
BugReports: https://github.com/fosterlab/PrInCE/issues
Description: PrInCE (Predicting Interactomes from Co-Elution) uses a naive
  Bayes classifier trained on dataset-derived features to recover
  protein-protein interactions from co-elution chromatogram profiles. This
  package contains the R implementation of PrInCE.
Depends: R (>= 3.6.0)
Imports: purrr (>= 0.2.4), dplyr (>= 0.7.4), tidyr (>= 0.8.99),
        forecast (>= 8.2), progress (>= 1.1.2), Hmisc (>= 4.0),
        naivebayes (>= 0.9.1), robustbase (>= 0.92-7), ranger (>=
        0.8.0), LiblineaR (>= 2.10-8), speedglm (>= 0.3-2), tester (>=
        0.1.7), magrittr (>= 1.5), Biobase (>= 2.40.0), MSnbase (>=
        2.8.3), stats, utils, methods, Rdpack (>= 0.7)
Suggests: BiocStyle, knitr, rmarkdown
biocViews: Proteomics, SystemsBiology, NetworkInference
VignetteBuilder: knitr
License: GPL-3 + file LICENSE
Encoding: UTF-8
RdMacros: Rdpack
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/PrInCE
git_branch: devel
git_last_commit: bfd792d
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 23:33:32 UTC; biocbuild
Author: Michael Skinnider [aut, trl, cre],
  R. Greg Stacey [ctb],
  Nichollas Scott [ctb],
  Anders Kristensen [ctb],
  Leonard Foster [aut, led]
Maintainer: Michael Skinnider <michael.skinnider@msl.ubc.ca>

# Generated by roxygen2: do not edit by hand

export(PrInCE)
export(adjacency_matrix_from_data_frame)
export(adjacency_matrix_from_list)
export(build_gaussians)
export(calculate_autocorrelation)
export(calculate_features)
export(calculate_precision)
export(check_gaussians)
export(choose_gaussians)
export(clean_profile)
export(clean_profiles)
export(co_apex)
export(concatenate_features)
export(detect_complexes)
export(filter_profiles)
export(fit_curve)
export(fit_gaussians)
export(impute_neighbors)
export(is_unweighted)
export(is_weighted)
export(make_feature_from_data_frame)
export(make_feature_from_expression)
export(make_initial_conditions)
export(make_labels)
export(match_matrix_dimensions)
export(predict_ensemble)
export(predict_interactions)
export(replace_missing_data)
export(threshold_precision)
importFrom(Biobase,"exprs<-")
importFrom(Hmisc,rcorr)
importFrom(LiblineaR,LiblineaR)
importFrom(MSnbase,exprs)
importFrom(Rdpack,reprompt)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,last)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n_distinct)
importFrom(dplyr,nth)
importFrom(dplyr,select)
importFrom(dplyr,starts_with)
importFrom(dplyr,ungroup)
importFrom(forecast,ma)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(naivebayes,naive_bayes)
importFrom(progress,progress_bar)
importFrom(purrr,map)
importFrom(purrr,map_dbl)
importFrom(purrr,map_dfr)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(ranger,ranger)
importFrom(robustbase,rowMedians)
importFrom(speedglm,speedglm)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nls)
importFrom(stats,predict)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(tester,is_logical_matrix)
importFrom(tester,is_numeric_matrix)
importFrom(tester,is_square_matrix)
importFrom(tidyr,crossing)
importFrom(utils,combn)

Prince
https://www.princexml.com

INSTALLATION

To install Prince, please run the install.sh shell script contained
in this distribution and follow the instructions provided.

The installation directory can be supplied as a command line argument:

    $ ./install.sh /path/to/directory

(You will need to be logged in as root if you wish to install into
a system directory such as /usr/local).

RUNNING PRINCE

To run Prince after installation, enter:

    $ prince input.html -o output.pdf

To see the usage and available options, enter:

    $ prince --help

If you have difficulty running the program, please ensure that the
installation directory is in your $PATH. This is necessary if you
did not install the program in /usr/local.

FEEDBACK

Thank you for choosing Prince, we hope you find it useful.
If you have any comments or questions regarding the program, please
email us or visit our website.

    Email:  prince@yeslogic.com
      Web:  https://www.princexml.com

## PrInCE

![PrInCE schematic](https://raw.githubusercontent.com/fosterlab/PrInCE/master/vignettes/schematic.png)

PrInCE is a computational approach to infer protein-protein interaction networks from co-elution proteomics data, also called co-migration, co-fractionation, or protein correlation profiling. This family of methods separates interacting protein complexes on the basis of their diameter or biochemical properties. Protein-protein interactions can then be inferred for pairs of proteins with similar elution profiles. PrInCE implements a machine-learning approach to identify protein-protein interactions given a set of labelled examples, using features derived exclusively from the data. This allows PrInCE to infer high-quality protein interaction networks from raw proteomics data, without bias towards known interactions or functionally associated proteins, making PrInCE a unique resource for discovery.

For a detailed introduction to PrInCE, see the vignette:

```
browseVignettes("PrInCE")
```

# Boehm-Demers-Weiser Garbage Collector

## Copyright & Warranty

 * Copyright (c) 1988, 1989 Hans-J. Boehm, Alan J. Demers
 * Copyright (c) 1991-1996 by Xerox Corporation.  All rights reserved.
 * Copyright (c) 1996-1999 by Silicon Graphics.  All rights reserved.
 * Copyright (c) 1999-2011 by Hewlett-Packard Development Company.

The files pthread_stop_world.c and pthread_support.c are also

 * Copyright (c) 1998 by Fergus Henderson.  All rights reserved.

THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.

Permission is hereby granted to use or copy this program
for any purpose,  provided the above notices are retained on all copies.
Permission to modify the code and to distribute modified code is granted,
provided the above notices are retained, and a notice that the code was
modified is included with the above copyright notice.

General Information
===================

LibThai is a set of Thai language support routines aimed to ease
developers' tasks to incorporate Thai language support in their
applications. It includes important Thai-specific functions e.g. word
breaking, input and output methods as well as basic character and
string supports. LibThai is an Open Source and collaborative effort
initiated by Thai Linux Working Group and opened for all contributors.

LibThai includes Thai language support routines:

  - Thai character set support: TIS 620, ISO/IEC 10646
  - Thai character properties
  - Thai string manipulators
  - Thai string collation
  - Thai word breaking
  - Thai cell clustering
  - Thai input method
  - Thai output method

More information about LibThai can be found at its official web site:
https://linux.thai.net/projects/libthai/ or via e-mail
thai-linux-foss-devel@googlegroups.com

Installation
============

See the file 'INSTALL'.

License
=======

LibThai is copyrighted by its authors and licensed under terms of the
GNU Lesser Public License (LGPL) - see file COPYING for details.

#' Make initial conditions for curve fitting with a mixture of Gaussians
#' 
#' Construct a set of initial conditions for curve fitting using nonlinear
#' least squares using a mixture of Gaussians. The "guess" method ports 
#' code from the Matlab release of PrInCE. This method finds local maxima 
#' within the chromatogram, orders them by their separation (in number of 
#' fractions) from the previous local maxima, and uses the positions and heights
#' of these local maxima (+/- some random noise) as initial conditions for 
#' Gaussian curve-fitting. The "random" method simply picks random values 
#' within the fraction and intensity intervals as starting points for Gaussian
#' curve-fitting. The initial value of sigma is set by default to a random 
#' number within +/- 0.5 of two for both modes; this is based on our manual 
#' inspection of a large number of chromatograms. 
#' 
#' @param chromatogram a numeric vector corresponding to the chromatogram trace
#' @param n_gaussians the number of Gaussians being fit 
#' @param method one of "guess" or "random", discussed above
#' @param sigma_default the default mean initial value of sigma
#' @param sigma_noise the amount of random noise to add or subtract from 
#' the default mean initial value of sigma
#' @param mu_noise the amount of random noise to add or subtract from the 
#' Gaussian centers in "guess" mode
#' @param A_noise the amount of random noise to add or subtract from the 
#' Gaussian heights in "guess" mode 
#' 
#' @return a list of three numeric vectors (A, mu, and sigma), each having
#' a length equal to the maximum number of Gaussians to fit 
#' 
#' @examples
#' data(scott)
#' chrom <- clean_profile(scott[16, ])
#' set.seed(0)
#' start <- make_initial_conditions(chrom, n_gaussians = 2, method = "guess")
#' 
#' @importFrom dplyr first last nth
#' 
#' @export
make_initial_conditions <- function(chromatogram, n_gaussians, 
                                    method = c("guess", "random"),
                                    sigma_default = 2, 
                                    sigma_noise = 0.5, mu_noise = 1.5, 
                                    A_noise = 0.5) {
  method <- match.arg(method)
  if (method == "guess") {
    # find fractions that represent local maxima
    peaksX <- which(diff(sign(diff(chromatogram))) == -2) + 1
    # catch local minima at start or end
    if (first(chromatogram) > nth(chromatogram, 2))
      peaksX <- c(peaksX, 1)
    if (last(chromatogram) > nth(chromatogram, -2))
      peaksX <- c(peaksX, length(chromatogram))
    # order local maxima by distance (on one side, only)
    ## distances <- diff(peaksX)
    ## peaksX <- peaksX[order(-distances)]
    # get intensities for sorted local maxima
    peaksY <- chromatogram[peaksX]
    # order by height
    peaksX <- peaksX[order(-peaksY)]
    # use local maxima as initial conditions for Gaussian fitting
    A <- numeric(0)
    mu <- numeric(0)
    sigma <- numeric(0)
    for (i in seq_len(n_gaussians)) {
      A[i] <- peaksY[i] + runif(1) - 0.5
      mu[i] <- peaksX[i] + runif(1, max = 3) - 1.5
      sigma[i] <- sigma_default + runif(1) - 0.5
    }
    # if there are not enough peaks, pick additional random values
    fractions <- length(chromatogram)
    A[is.na(A)] <- runif(sum(is.na(A)), max = fractions)
    mu[is.na(mu)] <- runif(sum(is.na(mu)), max = max(peaksY))
    initial_conditions <- list(A = A, mu = mu, sigma = sigma)
    return(initial_conditions)
  } else if (method == "random") {
    minHeight <- min(chromatogram)
    maxHeight <- max(chromatogram)
    A <- runif(n_gaussians, min = minHeight, max = maxHeight)
    mu <- runif(n_gaussians, min = 1, max = length(chromatogram))
    sigma <- sigma_default + runif(n_gaussians) - 0.5
    initial_conditions <- list(A = A, mu = mu, sigma = sigma)
    return(initial_conditions)
  }
}

#' Create a feature vector for a classifier from a data frame
#' 
#' Convert a data frame containing pairwise interactions, and  
#' a score or other data associated with each interaction, into a feature vector
#' that matches the dimensions of a data frame used as input to a classifier, 
#' such as a naive Bayes, random forests, or support vector machine classifier.
#' 
#' @param dat a data frame containing pairwise interactions and a feature to be
#'   converted to a vector in a third column
#' @param target the data frame of features that will be provided as input to a 
#'   classifier
#' @param dat_node_cols a vector of length two, denoting either the indices 
#'   (integer vector) or column names (character vector) of the columns within 
#'   the feature data frame; defaults to the first two columns of the data 
#'   frame (\code{c(1, 2)})
#' @param target_node_cols a vector of length two, denoting either the indices 
#'   (integer vector) or column names (character vector) of the columns within 
#'   the target data frame; defaults to the first two columns of the data 
#'   frame (\code{c(1, 2)})
#' @param feature_col the name or index of the column in the first data frame 
#'   that contains a feature for each interaction
#' @param default_value the default value for protein pairs that are not in the 
#'   first data frame (set, by default, to \code{NA})
#' 
#' @return a vector matching the dimensions and order of the feature data frame,
#' to use as input for a classifier in interaction prediction 
#' 
#' @export
make_feature_from_data_frame <- function(dat, 
                                         target, 
                                         dat_node_cols = c(1, 2),
                                         target_node_cols = c(1, 2),
                                         feature_col = 3, 
                                         default_value = NA) {
  # length of column names/indices must be exactly two (pairwise interactions)
  if (length(dat_node_cols) != 2) {
    stop("length of `dat_node_cols` must be exactly 2")
  }
  if (length(target_node_cols) != 2) {
    stop("length of `target_node_cols` must be exactly 2")
  }
  
  # get all proteins in feature data frame
  target_col1 <- target_node_cols[1]
  target_col2 <- target_node_cols[2]
  ref_proteins1 <- target[[target_col1]]
  ref_proteins2 <- target[[target_col2]]
  ref_proteins <- unique(c(ref_proteins1, ref_proteins2))
  
  # subset data frame to these proteins 
  query_col1 = dat_node_cols[1]
  query_col2 = dat_node_cols[2]
  overlap_idxs <- dat[[query_col1]] %in% ref_proteins & 
    dat[[query_col2]] %in% ref_proteins
  if (sum(overlap_idxs) == 0)
    stop("no proteins overlap between original and new features")
  filtered <- dat[overlap_idxs, ]
  
  # convert the first data frame to a matrix
  new_proteins1 <- filtered[[query_col1]]
  new_proteins2 <- filtered[[query_col2]] 
  new_proteins <- unique(c(new_proteins1, new_proteins2))
  feature_matrix <- matrix(default_value, nrow = length(new_proteins), 
                           ncol = length(new_proteins),
                           dimnames = list(new_proteins, new_proteins))
  
  # fill interaction scores
  feature_column <- filtered[[feature_col]]
  feature_matrix[as.matrix(filtered[, dat_node_cols])] <- feature_column
  
  # index matrix to get feature vector
  feat_idxs <- ref_proteins1 %in% rownames(feature_matrix) & 
    ref_proteins2 %in% rownames(feature_matrix)
  idxing_mat <- cbind(ref_proteins1[feat_idxs], ref_proteins2[feat_idxs])
  feature <- rep(default_value, nrow(target))
  feature[feat_idxs] <- feature_matrix[idxing_mat]
  return(feature)
}

#' Interactome of HeLa cells
#' 
#' Co-elution profiles derived from size exclusion chromatography (SEC) of
#' HeLa cell lysates.
#' 
#' Protein quantitation was accomplished by SILAC (stable isotopic labelling
#' by amino acids in cell culture), and is ratiometric, i.e., it reflects 
#' the ratio between the intensity of the heavy isotope and the light isotope 
#' ("H/L"). The dataset was initially described in Kristensen et al., 
#' \emph{Nat. Methods} 2012. The medium isotope channel from replicate
#' 1 (Supplementary Table 1a in the online supplementary information) is 
#' included in the \code{PrInCE} package. The R script used to generate
#' this matrix from the supplementary materials of the paper is provided in the
#' \code{data-raw} directory of the package source code.
#'
#' @docType data
#' @usage data(kristensen)
#' @format a data frame with 1875 rows and 48 columns, with proteins in rows and
#'   SEC fractions in columns
#' @source \url{https://www.nature.com/articles/nmeth.2131}
"kristensen"

#' Cytoplasmic interactome of Jurkat T cells during apoptosis
#' 
#' Co-elution profiles derived from size exclusion chromatography (SEC) of
#' cytoplasmic fractions from Jurkat T cells, 4 hours following Fas stimulation.
#' 
#' Protein quantitation was accomplished by SILAC (stable isotopic labelling
#' by amino acids in cell culture), and is ratiometric, i.e., it reflects 
#' the ratio between the intensity of the heavy isotope and the light isotope 
#' ("H/L"). The dataset was initially described in Scott et al., 
#' \emph{Mol. Syst. Biol.} 2017. The heavy isotope channel from replicate
#' 1 is included in the \code{PrInCE} package. The R script used to generate
#' this matrix from the supplementary materials of the paper is provided in the
#' \code{data-raw} directory of the package source code.
#'
#' @docType data
#' @usage data(scott)
#' @format a data frame with 1560 rows and 55 columns, with proteins in rows and
#'   SEC fractions in columns
#' @source \url{http://msb.embopress.org/content/13/1/906}
"scott"

#' Reference set of human protein complexes
#' 
#' A reference set of 467 experimentally confirmed human protein complexes,
#' derived from the EBI Complex Portal database. 
#' 
#' 467 protein complexes, ranging in size from 2 to 44 proteins and involving
#' 877 proteins in total, to provide a reference set of true positive and true 
#' negative interactions (intra- and inter-complex interactions, respectively)
#' for demonstration in PrInCE analysis of a co-elution dataset.
#' Other "gold standards" are possible in practice, most notably the CORUM
#' database; however, the Complex Portal reference set is included in this
#' package due to its CC-BY license. 
#' 
#' @docType data
#' @usage data(gold_standard)
#' @format a list containing 467 entries (character vectors)
#' @source \url{https://www.ebi.ac.uk/complexportal/complex/organisms}
"gold_standard"

#' Preprocess a co-elution profile
#' 
#' Clean a co-elution/co-fractionation profile by (1) imputing single missing
#' values with the average of neighboring values, (2) replacing missing values
#' with random, near-zero noise, and (3) smoothing with a moving average
#' filter. 
#' 
#' @param chromatogram a numeric vector corresponding to the chromatogram trace
#' @param impute_NA if true, impute single missing values with the average of
#' neighboring values 
#' @param smooth if true, smooth the chromatogram with a moving average filter
#' @param smooth_width width of the moving average filter, in fractions
#' @param noise_floor mean value of the near-zero noise to add  
#' 
#' @return a cleaned profile
#' 
#' @examples
#' data(scott)
#' chrom <- scott[16, ]
#' cleaned <- clean_profile(chrom)
#' 
#' @importFrom stats runif setNames
#' @importFrom forecast ma
#' 
#' @export
clean_profile <- function(chromatogram, impute_NA = TRUE, smooth = TRUE,
                          smooth_width = 4, noise_floor = 0.001) {
  # copy chromatogram
  cleaned <- chromatogram
    
  # impute missing values using mean of neighbors
  if (impute_NA) {
    impute_neighbors(cleaned)
  }
  
  # replace remaining NAs with near-zero noise
  nas <- is.na(cleaned) | cleaned == 0
  cleaned[nas] <- runif(sum(nas), min = 0, max = noise_floor)
  
  # smooth with a moving average filter
  if (smooth) {
    smooth_input <- c(runif(smooth_width, min = 0, max = noise_floor),
                      cleaned, 
                      runif(smooth_width, min = 0, max = noise_floor))
    smoothed <- ma(smooth_input, smooth_width)
    smoothed <- smoothed[seq(smooth_width + 1, length(smoothed) - smooth_width)]
    cleaned <- setNames(smoothed, names(cleaned))
  }
  
  return(cleaned)
}

#' Model selection for Gaussian mixture models
#' 
#' Calculate the AIC, corrected AIC, or BIC for a curve fit with a Gaussian 
#' mixture model by nonlinear least squares optimization. This function 
#' permits the calculation of the AIC/AICc/BIC after rejecting some Gaussians
#' in the model, for example because their centres are outside the bounds of 
#' the profile.
#'
#' @param coefs the coefficients of the Gaussian mixture model, output by 
#'   \code{\link{fit_gaussians}}
#' @param chromatogram the raw elution profile
#' 
#' @return the AIC, corrected AIC, or BIC of the fit model
#' 
#' @name aic
NULL

#' @rdname aic
gaussian_aic <- function(coefs, chromatogram) {
  # first, calculate log likelihood
  N <- length(chromatogram)
  indices <- seq_len(N)
  fit <- fit_curve(coefs, indices)
  res <- chromatogram - fit
  w <- rep_len(1, N)
  zw <- w == 0
  loglik <- -N * (log(2 * pi) + 1 - log(N) - sum(log(w + zw)) + 
                    log(sum(w * res^2)))/2
  # next, calculate parameters
  k <- length(unlist(coefs)) + 1
  AIC <- 2 * k - 2 * loglik
  return(AIC)
}

#' @rdname aic
gaussian_aicc <- function(coefs, chromatogram) {
  # first, calculate AIC
  AIC <- gaussian_aic(coefs, chromatogram)
  # second, calculate AICc
  N <- length(chromatogram)
  k <- length(unlist(coefs)) + 1
  AICc <- AIC + (2 * k * (k + 1)) / (N - k - 1)
  return(AICc)
}

#' @rdname aic
gaussian_bic <- function(coefs, chromatogram) {
  # first, calculate log likelihood
  N <- length(chromatogram)
  indices <- seq_len(N)
  fit <- fit_curve(coefs, indices)
  res <- chromatogram - fit
  w <- rep_len(1, N)
  zw <- w == 0
  loglik <- -N * (log(2 * pi) + 1 - log(N) - sum(log(w + zw)) + 
                    log(sum(w * res^2)))/2
  # next, calculate parameters
  k <- length(unlist(coefs)) + 1
  BIC <- log(N) * k - 2 * loglik
  return(BIC)
}

#' Fitted Gaussian mixture models for the \code{scott} dataset
#' 
#' The \code{\link[PrInCE]{scott}} dataset consists of protein
#' co-migration profiles derived from size exclusion chromatography (SEC) of
#' cytoplasmic fractions from Jurkat T cells, 4 hours following Fas stimulation.
#' The \code{scott_gaussians} object contains Gaussian mixture models fit by
#' the function \code{\link[PrInCE]{build_gaussians}}; this is bundled with the
#' R package in order to expedite the demonstration code, as the process of 
#' Gaussian fitting is one of the more time-consuming aspects of the package. 
#' 
#' As with the \code{\link[PrInCE]{scott}} dataset, the code used to generate
#' this data object is provided in the \code{data-raw} directory of the package
#' source.
#'
#' @docType data
#' @usage data(scott_gaussians)
#' @format a named list with 970 entries; names are proteins, and list items 
#'   conain information about fitted Gaussians in the format that PrInCE expects
"scott_gaussians"

#' Test whether a network is weighted
#'  
#' @param network the network to analyze 
#' 
#' @return true if the input network is a square numeric matrix with more than
#'   two values 
#' 
#' @examples 
#' data(gold_standard)
#' adj <- adjacency_matrix_from_list(gold_standard)
#' is_weighted(adj) ## returns FALSE
#' 
#' @importFrom tester is_square_matrix is_numeric_matrix
#' @importFrom dplyr n_distinct
#' 
#' @export
is_weighted <- function(network) {
  # check input is square integer, or numeric matrix
  square <- is_square_matrix(network)
  if (!square)
    stop("input is not a square matrix")
  dbl <- is_numeric_matrix(network)
  if (!dbl)
    stop("input could not be converted to numeric matrix")
  # convert to numeric 
  network <- network * 1
  # network should contain at least three different non-missing values
  n_distinct(c(network)) >= 3
}

#' Fit a Gaussian mixture model to a co-elution profile
#'
#' Fit mixtures of one or more Gaussians to the curve formed by a chromatogram
#' profile, and choose the best fitting model using an information criterion
#' of choice. 
#' 
#' @param chromatogram a numeric vector corresponding to the chromatogram trace
#' @param points optional, the number of non-NA points in the raw data
#' @param max_gaussians the maximum number of Gaussians to fit; defaults to 5.
#' Note that Gaussian mixtures with more parameters than observed (i.e., 
#' non-zero or NA) points will not be fit. 
#' @param criterion the criterion to use for model selection;
#'  one of "AICc" (corrected AIC, and default), "AIC", or "BIC"
#' @param max_iterations the number of times to try fitting the curve with
#' different initial conditions; defaults to 10
#' @param min_R_squared the minimum R-squared value to accept when fitting the
#' curve with different initial conditions; defaults to 0.5
#' @param method the method used to select the initial conditions for
#' nonlinear least squares optimization (one of "guess" or "random"); 
#' see \code{\link{make_initial_conditions}} for details
#' @param filter_gaussians_center true or false: filter Gaussians whose centres
#' fall outside the bounds of the chromatogram 
#' @param filter_gaussians_height Gaussians whose heights are below this 
#' fraction of the chromatogram height will be filtered. Setting this value to
#' zero disables height-based filtering of fit Gaussians
#' @param filter_gaussians_variance_min Gaussians whose variance is below this 
#' threshold will be filtered. Setting this value to zero disables filtering.
#' @param filter_gaussians_variance_max Gaussians whose variance is above this 
#' threshold will be filtered. Setting this value to zero disables filtering.
#'
#' @return a list with five entries: the number of Gaussians used to fit
#' the curve; the R^2 of the fit; the number of iterations used to 
#' fit the curve with different initial conditions; the coefficients of the 
#' fit model; and the curve predicted by the fit model.
#' 
#' @examples
#' data(scott)
#' chrom <- clean_profile(scott[1, ])
#' gauss <- choose_gaussians(chrom, max_gaussians = 3)
#' 
#' @importFrom purrr map_lgl map
#'
#' @export
choose_gaussians <- function(chromatogram, points = NULL,
                             max_gaussians = 5, 
                             criterion = c("AICc", "AIC", "BIC"),
                             max_iterations = 10, min_R_squared = 0.5,
                             method = c("guess", "random"),
                             filter_gaussians_center = TRUE,
                             filter_gaussians_height = 0.15,
                             filter_gaussians_variance_min = 0.1,
                             filter_gaussians_variance_max = 50) {
  criterion <- match.arg(criterion)
  
  # don't fit mixtures with more parameters than (experimental) points
  if (!is.null(points)) {
    max_gaussians <- min(max_gaussians, floor(points / 3))
  }
  
  # fit and choose models 
  fits <- list()
  for (n_gaussians in seq_len(max_gaussians))
    fits[[n_gaussians]] <- fit_gaussians(
      chromatogram, n_gaussians, max_iterations, min_R_squared,
      method = method, filter_gaussians_center, 
      filter_gaussians_height, filter_gaussians_variance_min,
      filter_gaussians_variance_max)
  
  # remove any models that failed to fit
  models <- map(fits, "coefs")
  drop <- map_lgl(models, is.null)
  fits <- fits[!drop]
  coefs <- map(fits, "coefs")
  
  if (criterion == "AICc") {
    criteria <- lapply(coefs, gaussian_aicc, chromatogram)
  } else if (criterion == "AIC") { 
    criteria <- lapply(coefs, gaussian_aic)
  } else if (criterion == "BIC") {
    criteria <- lapply(coefs, gaussian_bic)  
  }
  best <- which.min(criteria)
  if (length(best) == 0) {
    return(NULL)
  } else {
    return(fits[[best]])
  }
} 

#' Detect significantly interacting complexes in a chromatogram matrix
#' 
#' Use a permutation testing approach to identify complexes that show a 
#' significant tendency to interact, relative to random sets of complexes
#' of equivalent size. The function begins by calculating the Pearson 
#' correlation or Euclidean distance between all proteins in the matrix, and 
#' 
#' @param profile_matrix a matrix of chromatograms, with proteins in the rows
#'   and fractions in the columns, or a \code{\linkS4class{MSnSet}} object
#' @param complexes a named list of protein complexes, where the name is the
#'   complex name and the entries are proteins within that complex 
#' @param min_pairs the minimum number of pairwise observations to count a 
#'   correlation or distance towards the z score 
#' @param method method to use to calculate edge weights;
#'   one of \code{pearson} or \code{euclidean}
#' @param bootstraps number of bootstraps to execute to estimate z scores
#' @param progress whether to show the progress of the function
#' 
#' @return a named vector of z scores for each complex in the input list
#' 
#' @examples 
#' data(scott)
#' data(gold_standard)
#' complexes <- gold_standard[lengths(gold_standard) >= 3]
#' z_scores <- detect_complexes(t(scott), complexes)
#' length(na.omit(z_scores)) ## number of complexes that could be tested
#' z_scores[which.max(z_scores)] ## most significant complex
#' 
#' @importFrom stats cor dist na.omit median sd
#' @importFrom utils combn
#' @importFrom progress progress_bar
#' @importFrom purrr map_dbl
#' @importFrom MSnbase exprs
#' @importFrom methods is
#' 
#' @export
detect_complexes <- function(profile_matrix, complexes, 
                             method = c("pearson", "euclidean"),
                             min_pairs = 10, 
                             bootstraps = 100,
                             progress = TRUE) {
  method <- match.arg(method)
  
  if (is(profile_matrix, "MSnSet")) {
    profile_matrix <- exprs(profile_matrix)
  }
  
  # transpose for correlations
  profile_matrix = t(profile_matrix)
  
  # construct network
  n <- crossprod(!is.na(profile_matrix))
  if (method == "pearson") {
    network <- cor(profile_matrix, use = 'pairwise.complete.obs')
  } else if (method == "euclidean") {
    network <- as.matrix(dist(t(profile_matrix)))  
  }
  network[n < min_pairs] = NA
  
  # do permutation testing
  z_scores <- numeric(0)
  pb <- progress_bar$new(
    format = "complex :what [:bar] :percent eta: :eta",
    clear = FALSE, total = length(complexes), width = 80)
  for (i in seq_along(complexes)) {
    complex_name <- names(complexes)[i]
    complex <- complexes[[i]]
    
    # subset complex to proteins present in this network 
    nodes <- colnames(network)
    overlap <- intersect(complex, nodes)
    
    # abort if overlap size is < 3
    if (length(overlap) < 3) {
      z_scores[complex_name] <- NA
    } else {
      # calculate median PCC for intra-complex interactions
      idxing_mat <- t(combn(overlap, 2))
      edge_weights <- na.omit(network[idxing_mat])
      obs <- median(edge_weights, na.rm = TRUE)
      
      # compare to random expectation
      rnd <- map_dbl(seq_len(bootstraps), ~ {
        # generate random set of proteins equivalent to # of complex subunits
        # present in this network 
        rnd_complex <- sample(nodes, length(overlap))
        # calculate observed D statistic
        idxing_mat <- t(combn(rnd_complex, 2))
        rnd_weights <- as.numeric(na.omit(network[idxing_mat]))
        median(rnd_weights, na.rm = TRUE)
      })
      
      # calculate Z score
      z <- (obs - mean(rnd, na.rm = TRUE)) / sd(rnd, na.rm = TRUE)
      z_scores[complex_name] <- z
    }
    
    # tick progress bar
    if (progress) {
      pb$tick(tokens = list(what = sprintf(
        paste0("%-", nchar(length(complexes)), "s"), i)))
    }
  }
  
  return(z_scores)
}

#' Predict interactions given a set of features and examples
#' 
#' Discriminate interacting from non-interacting protein pairs by training a
#' machine learning model on a set of labelled examples, given a set of 
#' features derived from a co-elution profile matrix (see 
#' \code{\link[PrInCE]{calculate_features}}. 
#' 
#' PrInCE implements four different classifiers (naive Bayes, support vector
#' machine, random forest, and logistic regression). Naive Bayes is used as a
#' default. The classifiers are trained
#' on the gold standards using a ten-fold cross-validation procedure, training
#' on 90% of the data and predicting on the remaining 10%. For protein pairs
#' that are part of the training data, the held-out split is used to assign
#' a classifier score, whereas for the remaining protein pairs, the median of
#' all ten folds is used. Furthermore, to ensure the results are not sensitive
#' to the precise classifier split used, an ensemble of multiple classifiers
#' (ten, by default) is trained, and the classifier score is subsequently
#' averaged across classifiers. 
#' 
#' PrInCE can also ensemble across multiple different types of classifiers, 
#' by supplying the \code{"ensemble"} option to the \code{classifier} argument. 
#' 
#' @param features a data frame with proteins in the first two columns, and 
#'   features to be passed to the classifier in the remaining columns
#' @param gold_standard an adjacency matrix of "gold standard" interactions
#'  used to train the classifier 
#' @param classifier the type of classifier to use: one of \code{"NB"} (naive
#'   Bayes), \code{"SVM"} (support vector machine), \code{"RF"} (random forest),
#'   \code{"LR"} (logistic regression), or \code{"ensemble"} (an ensemble of
#'   all four)
#' @param verbose if \code{TRUE}, print a series of messages about the stage
#'   of the analysis
#' @param models the number of classifiers to train and average across, each
#'   with a different k-fold cross-validation split
#' @param cv_folds the number of folds to use for k-fold cross-validation
#' @param trees for random forests only, the number of trees in the forest
#' 
#' @return a ranked data frame of pairwise interactions, with the 
#' classifier score, label, and cumulative precision for each interaction 
#' 
#' @examples 
#' ## calculate features
#' data(scott)
#' data(scott_gaussians)
#' subset <- scott[seq_len(500), ] ## limit to first 500 proteins
#' gauss <- scott_gaussians[names(scott_gaussians) %in% rownames(subset)]
#' features <- calculate_features(subset, gauss)
#' ## load training data
#' data(gold_standard)
#' ref <- adjacency_matrix_from_list(gold_standard)
#' ## predict interactions
#' ppi <- predict_interactions(features, ref, cv_folds = 3, models = 1)
#' 
#' @importFrom dplyr starts_with group_by mutate_if mutate ungroup arrange
#'   full_join select
#' @importFrom magrittr %>%
#' 
#' @export
predict_interactions <- function(
  features, gold_standard, classifier = c("NB", "SVM", "RF", "LR", "ensemble"),
  verbose = FALSE, models = 10, cv_folds = 10, trees = 500) {
  classifier <- match.arg(classifier)
  
  ## define global variables to prevent check complaining
  protein_A <- NULL; protein_B <- NULL; score.x <- NULL; score.y <- NULL;
  score.x.x <- NULL; score.y.y <- NULL
  
  # make labels
  if (verbose) {
    message("making labels ...")
  }
  labels <- make_labels(gold_standard, features)
  
  if (classifier %in% c("NB", "SVM", "RF", "LR")) {
    if (verbose) {
      message("training classifiers ...")
    }
    # predict with a classifier ensemble
    interactions = predict_ensemble(features, 
                                    labels, 
                                    classifier = classifier, 
                                    models = models, 
                                    cv_folds = cv_folds, 
                                    trees = trees)
  } else if (classifier == "ensemble") {
    # predict all four separately 
    if (verbose) {
      message("training naive Bayes classifiers ...")
    }
    interactions_NB <- predict_ensemble(
      features, labels, classifier = "NB", models, cv_folds) 
    if (verbose) {
      message("training random forest classifiers ...")
    }
    interactions_RF <- predict_ensemble(
      features, labels, classifier = "RF", models, cv_folds, trees) 
    if (verbose) {
      message("training support vector machine classifiers ...")
    }
    interactions_SVM <- predict_ensemble(
      features, labels, classifier = "SVM", models, cv_folds)
    if (verbose) {
      message("training logistic regression classifiers ...")
    }
    interactions_LR <- predict_ensemble(
      features, labels, classifier = "LR", models, cv_folds)
    if (verbose) {
      message("ensembling predictions ...")
    }
    interactions_list <- list(interactions_NB, interactions_LR,
                             interactions_RF, interactions_SVM)
    interactions <- Reduce(function(x, y) full_join(
      x, y, by = c('protein_A', 'protein_B')), interactions_list) %>%
      select(-starts_with("label")) %>%
      mutate_if(is.numeric, ~ rank(-.)) %>%
      group_by(protein_A, protein_B) %>%
      mutate(mean = mean(c(score.x.x, score.y.y, score.x, score.y), 
                         na.rm = TRUE)) %>%
      ungroup() %>%
      arrange(mean)
    interactions$label <- make_labels(gold_standard, interactions)
  }
  
  # calculate precision
  interactions$precision <- calculate_precision(interactions$label)
  
  return(interactions)
}

#' Match the dimensions of a query matrix to a profile matrix
#' 
#' Match the row and column names of a square feature matrix to the row names
#' of a profile matrix, adding rows/columns containing \code{NA}s when 
#' proteins in the profile matrix are missing from the feature matrix. 
#' 
#' @param query a square matrix containing features for pairs of proteins 
#' @param profile_matrix the profile matrix for which interactions are being 
#' predicted 
#' 
#' @return a square matrix with the same row and column names as the input 
#' profile matrix, for use in interaction prediction 
#' 
#' @examples 
#' data(gold_standard)
#' subset <- adjacency_matrix_from_list(gold_standard[seq(1, 200)])
#' target <- adjacency_matrix_from_list(gold_standard)
#' matched <- match_matrix_dimensions(subset, target)
#' dim(subset)
#' dim(target)
#' dim(matched)
#' 
#' @export
match_matrix_dimensions <- function(query, profile_matrix) {
  proteins <- rownames(profile_matrix)
  # add empty rows and columns
  diff <- setdiff(proteins, colnames(query))
  new_rows <- matrix(NA, nrow = length(diff), ncol = ncol(query),
                     dimnames = list(diff, colnames(query)))
  query <- rbind(query, new_rows)
  new_cols <- matrix(NA, nrow = nrow(query), ncol = length(diff),
                     dimnames = list(rownames(query), diff))
  query <- cbind(query, new_cols)
  # same order as profile matrix
  query <- query[rownames(profile_matrix), rownames(profile_matrix)]
  return(query)
}

#' Calculate the autocorrelation for each protein between a pair of co-elution 
#' experiments.
#'
#' For a given protein, the correlation coefficient to all other proteins 
#' in the first condition is calculated, yielding a vector of correlation 
#' coefficients. The same procedure is repeated for the second condition,
#' and the two vectors of correlation coefficients are themselves correlated,
#' yielding a metric whereby higher values reflect proteins with unchanging
#' interaction profiles between conditions, while lower values reflect proteins 
#' with substantially changing interaction profiles.
#'
#' Note that all of zero, \code{NA}, \code{NaN}, and infinite values are
#' all treated equivalently as missing values when applying the 
#' \code{min_fractions} and \code{min_pairs} filters, but different handling of 
#' missing values will produce different autocorrelation scores.
#'
#' @param profile1 a numeric matrix or data frame with proteins in rows and 
#'   fractions in columns, or a \code{\linkS4class{MSnSet}} object, representing
#'   the first co-elution condition
#' @param profile2 a numeric matrix or data frame with proteins in rows and 
#'   fractions in columns, or a \code{\linkS4class{MSnSet}} object, representing
#'   the second co-elution condition
#' @param cor_method the correlation method to use; one of \code{"pearson"},
#'   \code{"spearman"}, or \code{"kendall"}).
#' @param min_fractions filter proteins not quantified in at least this many
#'   fractions
#' @param min_pairs remove correlations between protein pairs not co-occuring in
#'   at least this many fractions from the autocorrelation calculation
#'
#' @return a named vector of autocorrelation scores for all proteins found in
#' both matrices.
#'
#' @importFrom purrr map_dbl
#'
#' @export
calculate_autocorrelation <- function(profile1,
                                      profile2,
                                      cor_method = c("pearson", "spearman", 
                                                     "kendall"),
                                      min_replicates = 1,
                                      min_fractions = 1,
                                      min_pairs = 0) {
  cor_method <- match.arg(cor_method)
  
  # coerce profiles to matrices
  profile1 <- coerce_to_matrix(profile1)
  profile2 <- coerce_to_matrix(profile2)
  
  # identify 'missing' values as zeroes, NAs, or NaN
  missing1 <- !is.finite(profile1) | profile1 == 0
  missing2 <- !is.finite(profile2) | profile2 == 0
  
  # remove proteins that appear in less than min_fractions
  n_fractions1 <- rowSums(!missing1)
  n_fractions2 <- rowSums(!missing2)
  profile1 <- profile1[n_fractions1 >= min_fractions, ]
  profile2 <- profile2[n_fractions2 >= min_fractions, ]
  
  # filter to proteins found in both conditions
  overlap <- intersect(rownames(profile1), rownames(profile2))
  profile1 <- profile1[overlap, ]
  profile2 <- profile2[overlap, ]
  ## also filter missing
  missing1 <- missing1[overlap, ]
  missing2 <- missing2[overlap, ]
  
  # calculate correlations for each chromatogram
  cor1 <- cor(t(profile1), method = cor_method, use = 'pairwise.complete.obs')
  cor2 <- cor(t(profile2), method = cor_method, use = 'pairwise.complete.obs')
  
  # censor correlations with less than min_pairs in the raw chromatograms
  n1 <- crossprod(t(!missing1))
  n2 <- crossprod(t(!missing2))
  cor1[n1 < min_pairs] <- NA
  cor2[n2 < min_pairs] <- NA
  
  # calculate autocorrelations
  autocors <- map_dbl(overlap, ~ cor(cor1[., ], cor2[., ], 
                                     use = 'pairwise.complete.obs'))
  names(autocors) <- overlap
  return(autocors)
}

#' @importFrom MSnbase exprs
coerce_to_matrix <- function(matrix_like) {
  profile_matrix <- matrix_like
  if (is(matrix_like, "MSnSet")) {
    profile_matrix <- exprs(matrix_like)
  } else {
    profile_matrix <- data.matrix(matrix_like)
  }
  if (!is.numeric(profile_matrix)) {
    stop("input could not be coerced to numeric matrix: ",
         paste(dim(profile_matrix), collapse = ' '))
  }
  return(profile_matrix)
}

#' Threshold interactions at a given precision cutoff
#'
#' @param interactions the ranked list of interactions output by
#'   \code{\link[PrInCE]{predict_interactions}}, including a \code{precision}
#'   column
#' @param threshold the minimum precision of the unweighted interaction
#'   network to return
#'
#' @return the subset of the original ranked list at the given precision
#'
#' @examples
#' data(scott)
#' data(scott_gaussians)
#' data(gold_standard)
#' # analyze only the first 100 profiles
#' subset <- scott[seq_len(500), ]
#' gauss <- scott_gaussians[names(scott_gaussians) %in% rownames(subset)]
#' ppi <- PrInCE(subset, gold_standard,
#'   gaussians = gauss, models = 1,
#'   cv_folds = 3
#' )
#' network <- threshold_precision(ppi, threshold = 0.5)
#' nrow(network)
#' @export
threshold_precision <- function(interactions, threshold) {
  upto <- which(interactions$precision >= threshold)
  if (!length(upto)) {
    NULL
  } else {
    interactions[seq_len(max(upto, na.rm = T)), ]
  }
}

#' Calculate the co-apex score for every protein pair
#' 
#' Calculate the co-apex score for every pair of proteins. This is defined as 
#' the minimum Euclidean distance between any two Gaussians fit to each 
#' profile. 
#' 
#' @param gaussians a list of Gaussian mixture models fit to the profile matrix
#' by \code{link{build_gaussians}}
#' @param proteins all proteins being scored, optionally including those 
#' without Gaussian fits 
#' 
#' @return a matrix of co-apex scores 
#' 
#' @examples 
#' data(scott_gaussians)
#' gauss <- scott_gaussians[seq_len(25)]
#' CA <- co_apex(gauss)
#' 
#' @importFrom stats dist
#' @importFrom purrr map
#' 
#' @export
co_apex <- function(gaussians, proteins = NULL) {
  if (is.null(proteins)) {
    proteins <- names(gaussians)
  }
  n_proteins <- length(proteins)
  n_gaussians <- length(gaussians)
  gaussian_names <- names(gaussians)
  ## first, calculate Euclidean distance between every pair of Gaussians 
  gaussian_centers <- map(gaussians, c("coefs", "mu"))
  gaussian_sigmas <- map(gaussians, c("coefs", "sigma"))
  gaussian_matrix <- cbind(unlist(gaussian_centers), unlist(gaussian_sigmas))
  CA <- as.matrix(dist(gaussian_matrix))
  ## get indices for each protein 
  gaussian_indices <- rep(gaussian_names, lengths(gaussian_centers))
  ## calculate min co-apex score 
  co_apex <- matrix(NA, nrow = n_proteins, ncol = n_proteins,
                    dimnames = list(proteins, proteins))
  gaussian_names <- intersect(names(gaussians), proteins)
  for (protein_A in gaussian_names) {
    idxs_A <- which(gaussian_indices == protein_A)
    for (protein_B in gaussian_names) {
      idxs_B <- which(gaussian_indices == protein_B)
      co_apex[protein_A, protein_B] <- min(CA[idxs_A, idxs_B]) 
    }
  }
  return(co_apex)
}

#' Create an adjacency matrix from a data frame
#' 
#' Convert a data frame containing pairwise interactions into an
#' adjacency matrix. The resulting square adjacency matrix contains ones for 
#' proteins that are found in interactions and zeroes otherwise.
#' 
#' @param dat a data frame containing pairwise interactions
#' @param symmetric if true, interactions in both directions will be 
#'   added to the adjacency matrix
#' @param node_columns a vector of length two, denoting either the indices 
#'   (integer vector) or column names (character vector) of the columns within 
#'   the data frame containing the nodes participating in pairwise interactions;
#'   defaults to the first two columns of the data frame (\code{c(1, 2)})
#' 
#' @return an adjacency matrix between all interacting proteins
#' 
#' @examples
#' ppi <- data.frame(protein_A = paste0("protein", seq_len(10)),
#'                   protein_B = paste0("protein", c(rep(3, 2), rep(5, 5), 
#'                                      rep(7, 3))))
#' adj <- adjacency_matrix_from_data_frame(ppi)
#' 
#' @export
adjacency_matrix_from_data_frame <- function(dat, 
                                             symmetric = TRUE,
                                             node_columns = c(1, 2)
                                             ) {
  # length of node columns must be exactly two (pairwise interactions)
  if (length(node_columns) != 2) {
    stop("length of `node_columns` must be exactly 2")
  }
    
  # remove self-interactions
  col1 <- node_columns[1]
  col2 <- node_columns[2]
  dat <- dat[dat[[col1]] != dat[[col2]], ]
  nodes1 <- as.character(dat[[col1]])
  nodes2 <- as.character(dat[[col2]])
  
  # create empty matrix
  proteins <- unique(c(nodes1, nodes2))
  n_proteins <- length(proteins)
  adjacency <- matrix(0, nrow = n_proteins, ncol = n_proteins,
                      dimnames = list(proteins, proteins))
  
  # identify pairwise interactions
  adjacency[as.matrix(dat[, node_columns])] <- 1
  # add symmetric interactions
  if (symmetric) {
    adjacency[as.matrix(dat[, rev(node_columns)])] <- 1
  }
  return(adjacency)
}

#' PrInCE: Prediction of Interactomes from Co-Elution
#'
#' PrInCE is a computational approach to infer protein-protein interaction
#' networks from co-elution proteomics data, also called co-migration,
#' co-fractionation, or protein correlation profiling. This family of methods
#' separates interacting protein complexes on the basis of their diameter
#' or biochemical properties. Protein-protein interactions can then be
#' inferred for pairs of proteins with similar elution profiles. PrInCE
#' implements a machine-learning approach to identify protein-protein
#' interactions given a set of labelled examples, using features derived
#' exclusively from the data. This allows PrInCE to infer high-quality
#' protein interaction networks from raw proteomics data, without bias
#' towards known interactions or functionally associated proteins, making
#' PrInCE a unique resource for discovery.
#'
#' PrInCE takes as input a co-elution matrix, with detected proteins in rows and
#' fractions as columns, and a set of 'gold standard' true positives and true
#' negatives. If replicate experiments were performed, a list of co-elution
#' matrices can be provided as input. PrInCE will construct features for each
#' replicate separately and use features from all replicates as input to the
#' classifier. The 'gold standard' can be either a data frame or adjacency
#' matrix of known interactions (and non-interactions), or a list of protein
#' complexes. For computational convenience, Gaussian mixture models can be
#' pre-fit to every profile and provided separately to the \code{PrInCE}
#' function. The matrix, or matrices, can be provided to PrInCE either as
#' numeric matrices or as \code{\linkS4class{MSnSet}} objects.
#'
#' PrInCE implements three different types of classifiers to predict
#' protein-protein interaction networks, including naive Bayes (the default),
#' random forests, and support vector machines. The classifiers are trained
#' on the gold standards using a ten-fold cross-validation procedure, training
#' on 90% of the data and predicting on the remaining 10%. For protein pairs
#' that are part of the training data, the held-out split is used to assign
#' a classifier score, whereas for the remaining protein pairs, the median of
#' all ten folds is used. Furthermore, to ensure the results are not sensitive
#' to the precise classifier split used, an ensemble of multiple classifiers
#' (ten, by default) is trained, and the classifier score is subsequently
#' averaged across classifiers. PrInCE can also ensemble across a set of
#' classifiers.
#'
#' By default, PrInCE calculates six features from each pair of co-elution
#' profiles as input to the classifier, including conventional similarity
#' metrics but also several features specifically adapted to co-elution
#' proteomics. For example, one such feature is derived
#' from fitting a Gaussian mixture model to each elution profile, then
#' calculating the smallest Euclidean distance between any pair of fitted
#' Gaussians. The complete set of features includes:
#'
#' \enumerate{
#'   \item the Pearson correlation between raw co-elution profiles;
#'   \item the p-value of the Pearson correlation between raw co-elution
#'     profiles;
#'   \item the Pearson correlation between cleaned profiles, which are generated
#'     by imputing single missing values with the mean of their neighbors,
#'     replacing remaining missing values with random near-zero noise, and
#'     smoothing the profiles using a moving average filter (see
#'     \code{\link[PrInCE]{clean_profile}});
#'  \item the Euclidean distance between cleaned profiles;
#'  \item the 'co-peak' score, defined as the distance, in fractions, between
#'    the maximum values of each profile; and
#'  \item the 'co-apex' score, defined as the minimum Euclidean distance between
#'    any pair of fit Gaussians
#' }
#'
#' The output of PrInCE is a ranked data frame, containing the classifier score
#' for every possible protein pair. PrInCE also calculates the precision at
#' every point in this ranked list, using the 'gold standard' set of protein
#' complexes or binary interactions. Our recommendation is to select a
#' threshold for the precision and use this to construct an unweighted
#' protein interaction network.
#'
#' @param profiles the co-elution profile matrix, or a list of profile matrices
#'   if replicate experiments were performed. Can be a single numeric matrix,
#'   with proteins in rows and fractions in columns, or a list of matrices.
#'   Alternatively, can be provided as a single
#'   \code{\linkS4class{MSnSet}} object or a list of objects.
#' @param gold_standard a set of 'gold standard' interactions, used to train the
#'   classifier. Can be provided either as an adjacency matrix, in which
#'   both rows and columns correspond to protein IDs in the co-elution matrix
#'   or matrices, or as a list of proteins in the same complex, which will be
#'   converted to an adjacency matrix by PrInCE. Zeroes in the adjacency matrix
#'   are interpreted by PrInCE as "true negatives" when calculating precision.
#' @param gaussians optionally, provide Gaussian mixture models fit by
#'   the \code{\link[PrInCE]{build_gaussians}} function. If \code{profiles} is
#'   a numeric matrix, this should be the named list output by
#'   \code{\link[PrInCE]{build_gaussians}} for that matrix; if \code{profiles}
#'   is a list of numeric matrices, this should be a list of named lists
#' @param precision optionally, return only interactions above the given
#'   precision; by default, all interactions are returned and the user can
#'   subsequently threshold the list using the
#'   \code{\link[PrInCE]{threshold_precision}} function
#' @param verbose if \code{TRUE}, print a series of messages about the stage
#'   of the analysis
#' @param min_points filter profiles without at least this many total,
#'   non-missing points; passed to \code{\link{filter_profiles}}
#' @param min_consecutive filter profiles without at least this many
#'   consecutive, non-missing points; passed to \code{\link{filter_profiles}}
#' @param min_pairs minimum number of overlapping fractions between any given
#'   protein pair to consider a potential interaction
#' @param impute_NA if true, impute single missing values with the average of
#'   neighboring values; passed to \code{\link{clean_profiles}}
#' @param smooth if true, smooth the chromatogram with a moving average filter;
#'   passed to \code{\link{clean_profiles}}
#' @param smooth_width width of the moving average filter, in fractions;
#'   passed to \code{\link{clean_profiles}}
#' @param max_gaussians the maximum number of Gaussians to fit; defaults to 5.
#'   Note that Gaussian mixtures with more parameters than observed (i.e.,
#'   non-zero or NA) points will not be fit. Passed to
#'   \code{\link{choose_gaussians}}
#' @param criterion the criterion to use for model selection;
#'   one of "AICc" (corrected AIC, and default), "AIC", or "BIC". Passed to
#'   \code{\link{choose_gaussians}}
#' @param max_iterations the number of times to try fitting the curve with
#'   different initial conditions; defaults to 50. Passed to
#'   \code{\link{fit_gaussians}}
#' @param min_R_squared the minimum R-squared value to accept when fitting the
#'   curve with different initial conditions; defaults to 0.5. Passed to
#'   \code{\link{fit_gaussians}}
#' @param method the method used to select the initial conditions for
#'   nonlinear least squares optimization (one of "guess" or "random");
#'   see \code{\link{make_initial_conditions}} for details. Passed to
#'   \code{\link{fit_gaussians}}
#' @param pearson_R_raw if true, include the Pearson correlation (R) between
#'   raw profiles as a feature
#' @param pearson_R_cleaned if true, include the Pearson correlation (R) between
#'   cleaned profiles as a feature
#' @param pearson_P if true, include the P-value of the Pearson correlation
#'   between raw profiles as a feature
#' @param euclidean_distance if true, include the Euclidean distance between
#'   cleaned profiles as a feature
#' @param co_peak if true, include the 'co-peak score' (that is, the distance,
#'   in fractions, between the single highest value of each profile) as a
#'   feature
#' @param co_apex if true, include the 'co-apex score' (that is, the minimum
#'   Euclidean distance between any pair of fit Gaussians) as a feature
#' @param n_pairs if \code{TRUE}, include the number of fractions in which
#'   both of a given pair of proteins were detected as a feature
#' @param classifier the type of classifier to use: one of \code{"NB"} (naive
#'   Bayes), \code{"SVM"} (support vector machine), \code{"RF"} (random forest),
#'   \code{"LR"} (logistic regression), or \code{"ensemble"} (an ensemble of
#'   all four)
#' @param models the number of classifiers to train and average across, each
#'   with a different k-fold cross-validation split
#' @param cv_folds the number of folds to use for k-fold cross-validation
#' @param trees for random forests only, the number of trees in the forest
#'
#' @return a ranked data frame of interacting proteins, with the precision
#'   at each point in the list
#'
#' @examples
#' data(scott)
#' data(scott_gaussians)
#' data(gold_standard)
#' # analyze only the first 100 profiles
#' subset <- scott[seq_len(500), ]
#' gauss <- scott_gaussians[names(scott_gaussians) %in% rownames(subset)]
#' ppi <- PrInCE(subset, gold_standard,
#'   gaussians = gauss, models = 1,
#'   cv_folds = 3
#' )
#' @importFrom Rdpack reprompt
#' @importFrom MSnbase exprs
#' @importFrom methods is
#'
#' @export
#'
#' @references
#' \insertRef{stacey2017}{PrInCE}
#'
#' \insertRef{scott2015}{PrInCE}
#'
#' \insertRef{kristensen2012}{PrInCE}
#'
#' \insertRef{skinnider2018}{PrInCE}
PrInCE <- function(profiles, gold_standard,
                   gaussians = NULL,
                   precision = NULL,
                   verbose = FALSE,
                   ## build_gaussians
                   min_points = 1,
                   min_consecutive = 5,
                   min_pairs = 3,
                   impute_NA = TRUE,
                   smooth = TRUE,
                   smooth_width = 4,
                   max_gaussians = 5,
                   max_iterations = 50,
                   min_R_squared = 0.5,
                   method = c("guess", "random"),
                   criterion = c("AICc", "AIC", "BIC"),
                   ## calculate_features
                   pearson_R_raw = TRUE,
                   pearson_R_cleaned = TRUE,
                   pearson_P = TRUE,
                   euclidean_distance = TRUE,
                   co_peak = TRUE,
                   co_apex = TRUE,
                   n_pairs = FALSE,
                   ## predict_interactions
                   classifier = c("NB", "SVM", "RF", "LR", "ensemble"),
                   models = 1,
                   cv_folds = 10,
                   trees = 500) {
  method <- match.arg(method)
  criterion <- match.arg(criterion)
  classifier <- match.arg(classifier)

  # check profile input
  if (is.list(profiles) & !is.data.frame(profiles)) {
    for (replicate_idx in seq_along(profiles)) {
      replicate <- profiles[[replicate_idx]]
      if (is(replicate, "MSnSet")) {
        profile_matrix <- exprs(replicate)
      } else {
        profile_matrix <- data.matrix(replicate)
      }
      if (!is.numeric(profile_matrix)) {
        stop(
          "list input (item #", replicate_idx,
          ") could not be coerced to numeric matrix"
        )
      }
      profiles[[replicate_idx]] <- profile_matrix
      # also check Gaussians
      if (!is.null(gaussians)) {
        if (length(gaussians) < replicate_idx) {
          stop("fewer Gaussians than profiles provided")
        }
        check_gaussians(
          gaussians[[replicate_idx]], rownames(profile_matrix),
          replicate_idx
        )
      }
    }
  } else {
    if (is(profiles, "MSnSet")) {
      profile_matrix <- exprs(profiles)
    } else {
      profile_matrix <- data.matrix(profiles)
    }
    if (!is.numeric(profile_matrix)) {
      stop("input could not be coerced to numeric matrix")
    }
    # wrap in a list
    profiles <- list(profile_matrix)
    # also check Gaussians
    if (!is.null(gaussians)) {
      check_gaussians(gaussians)
      gaussians <- list(gaussians)
    }
  }

  # check gold standard input
  if (is.data.frame(gold_standard)) {
    # convert to adjacency matrix
    gold_standard <- adjacency_matrix_from_data_frame(gold_standard)
  } else if (is.list(gold_standard)) {
    gold_standard <- adjacency_matrix_from_list(gold_standard)
  }
  if (!is_unweighted(gold_standard)) {
    stop("could not convert supplied gold standards to adjacency matrix")
  }

  # get features for each matrix separately
  features <- list()
  for (replicate_idx in seq_along(profiles)) {
    if (verbose) {
      message("generating features for replicate ", replicate_idx, " ...")
    }
    mat <- profiles[[replicate_idx]]

    # read Gaussians, or fit if they haven't been yet
    if (!is.null(gaussians)) {
      gauss <- gaussians[[replicate_idx]]
    } else {
      if (verbose) {
        message("  fitting Gaussians ...")
      }
      gauss <- build_gaussians(mat,
        min_points = min_points,
        min_consecutive = min_consecutive,
        impute_NA = impute_NA,
        smooth = smooth,
        smooth_width = smooth_width,
        max_gaussians = max_gaussians,
        max_iterations = max_iterations,
        min_R_squared = min_R_squared,
        method = method
      )
    }

    # filter matrix based on Gaussians
    before <- nrow(mat)
    mat <- mat[names(gauss), ]
    after <- nrow(mat)
    if (verbose) {
      message(
        "  fit mixtures of Gaussians to ", after, " of ", before,
        " profiles"
      )
    }

    # calculate features
    feat <- calculate_features(mat, gauss,
      min_pairs = min_pairs,
      pearson_R_raw = pearson_R_raw,
      pearson_R_cleaned = pearson_R_cleaned,
      pearson_P = pearson_P,
      euclidean_distance = euclidean_distance,
      co_peak = co_peak,
      co_apex = co_apex,
      n_pairs = n_pairs
    )
    features[[replicate_idx]] <- feat
  }

  # collapse into a single data frame
  if (verbose) {
    message("concatenating features across replicates ...")
  }
  input <- concatenate_features(features)

  # impute missing values
  input <- replace_missing_data(input)

  # predict interactions
  interactions <- predict_interactions(input, gold_standard,
    classifier = classifier,
    models = models,
    cv_folds = cv_folds,
    trees = trees,
    verbose = verbose
  )

  # optionally threshold based on precison
  if (!is.null(precision)) {
    before <- nrow(interactions)
    interactions <- threshold_precision(interactions, precision)
    if (is.null(interactions)) {
      warning(
        "none of ", before, " ranked protein pairs had precision >= ",
        precision
      )
    }
  }

  return(interactions)
}

#' Calculate the default features used to predict interactions in PrInCE
#' 
#' Calculate the six features that are used to discriminate interacting and
#' non-interacting protein pairs based on co-elution profiles in PrInCE, 
#' namely: raw Pearson R value, cleaned Pearson R value, raw Pearson P-value,
#' Euclidean distance, co-peak, and co-apex. Optionally, one or more of these
#' can be disabled.
#' 
#' @param profile_matrix a numeric matrix of co-elution profiles, with proteins
#' in rows, or a \code{\linkS4class{MSnSet}} object
#' @param gaussians a list of Gaussian mixture models fit to the profile matrix
#' by \code{link{build_gaussians}}
#' @param min_pairs minimum number of overlapping fractions between any given
#'   protein pair to consider a potential interaction
#' @param pearson_R_raw if true, include the Pearson correlation (R) between
#' raw profiles as a feature
#' @param pearson_R_cleaned if true, include the Pearson correlation (R) between
#' cleaned profiles as a feature
#' @param pearson_P if true, include the P-value of the Pearson correlation 
#' between raw profiles as a feature
#' @param euclidean_distance if true, include the Euclidean distance between
#' cleaned profiles as a feature
#' @param co_peak if true, include the 'co-peak score' (that is, the distance,
#' in fractions, between the single highest value of each profile) as a feature 
#' @param co_apex if true, include the 'co-apex score' (that is, the minimum
#' Euclidean distance between any pair of fit Gaussians) as a feature
#' @param max_euclidean_quantile very high Euclidean distance values are trimmed
#' to avoid numerical precision issues; values above this quantile will be
#' replaced with the value at this quantile (default: \code{0.9})
#' 
#' @return a data frame containing the calculated features for all possible
#' protein pairs
#' 
#' @importFrom stats cor dist
#' @importFrom MSnbase exprs
#' @importFrom methods is
#' @importFrom Hmisc rcorr
#' @importFrom purrr map map_int
#' @importFrom dplyr filter
#' 
#' @export
calculate_features <- function(profile_matrix, gaussians,
                               min_pairs = 0,
                               pearson_R_raw = TRUE,
                               pearson_R_cleaned = TRUE,
                               pearson_P = TRUE,
                               euclidean_distance = TRUE,
                               co_peak = TRUE,
                               co_apex = TRUE,
                               n_pairs = FALSE,
                               max_euclidean_quantile = 0.9
) {
  if (is(profile_matrix, "MSnSet")) {
    profile_matrix <- exprs(profile_matrix)
  }
  # remove at least pairs supported by only 2 points
  if (min_pairs < 3) {
    min_pairs <- 3
  }
  
  # replace missing values with near-zero noise
  cleaned <- clean_profiles(profile_matrix, impute_NA = FALSE, smooth = FALSE,
                            noise_floor = 0.05)
  proteins <- rownames(cleaned)
  n_proteins <- length(proteins)
  
  # create features list 
  feature_matrices <- list() 
  ## cor_R_raw, cor_R_cleaned, cor_P, eucl, co_peak, CA)
  
  # calculate Pearson correlation and P-value distances
  pairs <- crossprod(t(!is.na(profile_matrix)))
  if (pearson_R_raw) {
    cor_R_raw <- suppressWarnings(
      1 - cor(t(profile_matrix), use = 'pairwise.complete.obs'))
    feature_matrices[["cor_R_raw"]] <- cor_R_raw
  }
  if (pearson_R_cleaned) {
    cor_R_cleaned <- 1 - cor(t(cleaned))
    feature_matrices[["cor_R_cleaned"]] <- cor_R_cleaned
  }
  if (pearson_P) {
    cor_P <- suppressWarnings(
      rcorr(t(profile_matrix))$P)
    feature_matrices[["cor_P"]] <- cor_P  
  }
  # calculate Euclidean distance
  if (euclidean_distance) {
    eucl <- as.matrix(dist(cleaned, method = 'euclidean'))
    feature_matrices[["euclidean_distance"]] <- eucl
  }
  # calculate co-peak distance
  if (co_peak) {
    maxes <- apply(cleaned, 1, which.max)
    co_peak <- as.matrix(dist(maxes))
    feature_matrices[["co_peak"]] <- co_peak
  }
  # calculate co-apex (Gaussian) score
  if (co_apex) {
    CA <- co_apex(gaussians, proteins)
    feature_matrices[["co_apex"]] <- CA
  }
  
  # add number of pairs to the feature matrix list
  feature_matrices[["n_pairs"]] <- pairs
  
  ## make sure all dimensions are identical
  if (!all(map_int(feature_matrices, nrow) == n_proteins) |
      !all(map_int(feature_matrices, ncol) == n_proteins))
    stop("at least one feature matrix did not have correct dimensions")
  if (length(feature_matrices) == 0)
    stop("no features were calculated")
  first <- feature_matrices[[1]]
  tri <- upper.tri(first)
  idxs <- which(tri, arr.ind = TRUE)
  dat <- data.frame(protein_A = rownames(first)[idxs[, 1]], 
                    protein_B = rownames(first)[idxs[, 2]],
                    stringsAsFactors = FALSE) 
  dat <- cbind(dat, map(feature_matrices, ~ .[tri]))
  
  ## cap Euclidean distance at the 99.5th %ile
  if ("euclidean_distance" %in% colnames(dat)) {
    vec = dat$euclidean_distance
    threshold = quantile(vec, probs = max_euclidean_quantile, na.rm = T)
    dat$euclidean_distance[vec >= threshold] <- threshold
  }
  
  # filter features based on n_pairs
  dat <- filter(dat, n_pairs >= min_pairs)
  # remove n_pairs from the feature list
  if (!n_pairs) {
    dat$n_pairs <- NULL
  }
  
  return(dat)
}

#' Create a feature vector from expression data
#' 
#' Convert a gene or protein expression matrix into a feature vector
#' that matches the dimensions of a data frame used as input to a classifier, 
#' such as a naive Bayes, random forests, or support vector machine classifier,
#' by calculating the correlation between each pair of genes or proteins.  
#' 
#' @param expr a matrix containing gene or protein expression data, with 
#'   genes/proteins in columns and samples in rows
#' @param dat the data frame of features to be used by the classifier, 
#'    with protein pairs in the columns specified by the \code{node_columns}
#'    argument
#' @param node_columns a vector of length two, denoting either the indices 
#'   (integer vector) or column names (character vector) of the columns within 
#'   the data frame containing the nodes participating in pairwise interactions;
#'   defaults to the first two columns of the data frame (\code{c(1, 2)})
#' @param ... arguments passed to \code{cor}
#' 
#' @return a vector matching the dimensions and order of the feature data frame,
#' to use as input for a classifier in interaction prediction 
#' 
#' @export
make_feature_from_expression <- function(expr, dat, 
                                         node_columns = c(1, 2),
                                         ...) {
  # length of node columns must be exactly two (pairwise interactions)
  if (length(node_columns) != 2) {
    stop("length of `node_columns` must be exactly 2")
  }
  
  # get all proteins in feature data frame
  col1 <- node_columns[1]
  col2 <- node_columns[2]
  proteins_1 <- dat[[col1]]
  proteins_2 <- dat[[col2]]
  proteins <- unique(c(proteins_1, proteins_2))
  
  # subset expression to these proteins 
  filtered <- 1.0 * expr[, colnames(expr) %in% proteins]
  if (ncol(filtered) == 0)
    stop("no proteins overlap between expression and feature data")
  
  # create coexpression matrix
  coexpr <- cor(filtered, ...)
  
  # index matrix to get feature vector
  feat_idxs <- proteins_1 %in% rownames(coexpr) & 
    proteins_2 %in% rownames(coexpr)
  idxing_mat <- cbind(proteins_1[feat_idxs], proteins_2[feat_idxs])
  feature <- rep(NA, nrow(dat))
  feature[feat_idxs] <- coexpr[idxing_mat]
  return(feature)
}

#' Deconvolve profiles into Gaussian mixture models
#' 
#' Identify peaks in co-fractionation profiles by deconvolving peaks in 
#' Gaussian mixture models. Models are mixtures of between 1 and 5 Gaussians.
#' Profiles are pre-processed prior to building Gaussians by filtering and 
#' cleaning. By default, profiles with fewer than 5 non-missing points, or 
#' fewer than 5 consecutive points after imputation of single missing values,
#' are removed. Profiles are cleaned by replacing missing values with 
#' near-zero noise, imputing single missing values as the mean of neighboring 
#' points, and smoothing with a moving average filter. 
#' 
#' @param profile_matrix a numeric matrix of co-elution profiles, with proteins
#' in rows, or a \code{\linkS4class{MSnSet}} object
#' @param min_points filter profiles without at least this many total, 
#' non-missing points; passed to \code{\link{filter_profiles}}
#' @param min_consecutive filter profiles without at least this many 
#' consecutive, non-missing points; passed to \code{\link{filter_profiles}}
#' @param impute_NA if true, impute single missing values with the average of
#' neighboring values; passed to \code{\link{clean_profiles}}
#' @param smooth if true, smooth the chromatogram with a moving average filter;
#' passed to \code{\link{clean_profiles}}
#' @param smooth_width width of the moving average filter, in fractions;
#' passed to \code{\link{clean_profiles}}
#' @param max_gaussians the maximum number of Gaussians to fit; defaults to 5.
#' Note that Gaussian mixtures with more parameters than observed (i.e., 
#' non-zero or NA) points will not be fit. Passed to  
#' \code{\link{choose_gaussians}}
#' @param criterion the criterion to use for model selection;
#' one of "AICc" (corrected AIC, and default), "AIC", or "BIC". Passed to
#' \code{\link{choose_gaussians}}
#' @param max_iterations the number of times to try fitting the curve with
#' different initial conditions; defaults to 50. Passed to 
#' \code{\link{fit_gaussians}}
#' @param min_R_squared the minimum R-squared value to accept when fitting the
#' curve with different initial conditions; defaults to 0.5. Passed to 
#' \code{\link{fit_gaussians}}
#' @param method the method used to select the initial conditions for
#' nonlinear least squares optimization (one of "guess" or "random"); 
#' see \code{\link{make_initial_conditions}} for details. Passed to 
#' \code{\link{fit_gaussians}}
#' @param filter_gaussians_center true or false: filter Gaussians whose centres
#' fall outside the bounds of the chromatogram. Passed to 
#' \code{\link{fit_gaussians}}
#' @param filter_gaussians_height Gaussians whose heights are below this 
#' fraction of the chromatogram height will be filtered. Setting this value to
#' zero disables height-based filtering of fit Gaussians. Passed to 
#' \code{\link{fit_gaussians}}
#' @param filter_gaussians_variance_min Gaussians whose variance falls below 
#' this number of fractions will be filtered. Setting this value to
#' zero disables filtering. Passed to 
#' \code{\link{fit_gaussians}}
#' @param filter_gaussians_variance_max Gaussians whose variance is above
#' this number of fractions will be filtered. Setting this value to
#' zero disables filtering. Passed to 
#' \code{\link{fit_gaussians}}
#' 
#' @return a list of fit Gaussian mixture models, where each item in the 
#' list contains the following five fields: the number of Gaussians used to fit
#' the curve; the R^2 of the fit; the number of iterations used to 
#' fit the curve with different initial conditions; the coefficients of the 
#' fit model; and the curve predicted by the fit model. Profiles that 
#' could not be fit by a Gaussian mixture model above the minimum R-squared 
#' cutoff will be absent from the returned list.
#' 
#' @examples
#' data(scott)
#' mat <- clean_profiles(scott[seq_len(5), ])
#' gauss <- build_gaussians(mat, max_gaussians = 3)
#' 
#' @importFrom MSnbase exprs
#' @importFrom methods is
#' @importFrom progress progress_bar
#' 
#' @export
build_gaussians <- function(profile_matrix, 
                            min_points = 1, min_consecutive = 5,
                            impute_NA = TRUE, smooth = TRUE, smooth_width = 4,
                            max_gaussians = 5, 
                            criterion = c("AICc", "AIC", "BIC"),
                            max_iterations = 50, min_R_squared = 0.5,
                            method = c("guess", "random"),
                            filter_gaussians_center = TRUE,
                            filter_gaussians_height = 0.15,
                            filter_gaussians_variance_min = 0.5,
                            filter_gaussians_variance_max = 50) {
  if (is(profile_matrix, "MSnSet")) {
    profile_matrix <- exprs(profile_matrix)
  }
  
  # preprocess chromatograms: filter and clean
  filtered <- filter_profiles(profile_matrix,
                              min_points = min_points,
                              min_consecutive = min_consecutive)
  cleaned <- clean_profiles(filtered,
                            impute_NA = impute_NA,
                            smooth = smooth,
                            smooth_width = smooth_width)
  
  # fit Gaussians, displaying progress bar
  gaussians <- list()
  proteins <- rownames(cleaned)
  P <- length(proteins)
  message(".. fitting Gaussian mixture models to ", P, " profiles")
  pb <- progress_bar$new(
    format = "fitting :what [:bar] :percent eta: :eta",
    clear = FALSE, total = P, width = 80)
  max_len <- max(nchar(proteins))
  for (i in seq_len(P)) {
    protein <- proteins[i]
    pb$tick(tokens = list(what = sprintf(paste0("%-", max_len, "s"), protein)))
    chromatogram <- cleaned[protein,]
    points <- sum(!is.na(profile_matrix[protein,]))
    gaussian <- choose_gaussians(chromatogram, points,
                                 max_gaussians, criterion,
                                 max_iterations, min_R_squared,
                                 method, filter_gaussians_center,
                                 filter_gaussians_height,
                                 filter_gaussians_variance_min,
                                 filter_gaussians_variance_max)
    gaussians[[protein]] <- gaussian
  }
  
  return(gaussians)
}

#' Replace missing data with median ± random noise 
#' 
#' Replace missing data within each numeric column of a data frame with 
#' the column median, plus or minus some random noise, in order to train 
#' classifiers that do not easily ignore missing data (e.g. random forests or
#' support vector machines).
#' 
#' @param dat the data frame to replace missing data in 
#' @param noise_pct the standard deviation of the random normal 
#' distribution from which to draw added noise, expressed as a 
#' percentage of the standard deviation of the non-missing values in each 
#' column
#' 
#' @return a data frame with missing values in each numeric column replaced
#' by the column median, plus or minus some random noise
#' 
#' @importFrom stats rnorm
#' 
#' @export
replace_missing_data <- function(dat, noise_pct = 0.05) {
  for (col_name in colnames(dat)) {
    column <- dat[[col_name]]
    if (!is.numeric(column))
      next
    ## first, replace infinite values
    infinite = is.infinite(column)
    column[infinite] <- NA
    ## second, replace other missing values
    missing <- !is.finite(column)
    dat[[col_name]][missing] <- 
      median(column, na.rm = TRUE) + rnorm(sum(missing)) * 
      sd(column, na.rm = TRUE) * noise_pct 
  }
  return(dat)
}

#' Fitted Gaussian mixture models for the \code{kristensen} dataset
#' 
#' The \code{\link[PrInCE]{kristensen}} dataset consists of protein
#' co-migration profiles derived from size exclusion chromatography (SEC) of
#' unstimulated HeLa cell lysates. 
#' The \code{kristensen_gaussians} object contains Gaussian mixture models fit by
#' the function \code{\link[PrInCE]{build_gaussians}}; this is bundled with the
#' R package in order to expedite the demonstration code, as the process of 
#' Gaussian fitting is one of the more time-consuming aspects of the package. 
#' 
#' As with the \code{\link[PrInCE]{kristensen}} dataset, the code used to
#' generate this data object is provided in the \code{data-raw} directory of the
#' package source.
#'
#' @docType data
#' @usage data(kristensen_gaussians)
#' @format a named list with 1117 entries; names are proteins, and list items 
#'   conain information about fitted Gaussians in the format that PrInCE expects
"kristensen_gaussians"

#' Create an adjacency matrix from a list of complexes
#' 
#' Convert a list of complexes into a pairwise adjacency matrix. The resulting
#' square adjacency matrix contains ones for proteins that are found in the
#' same complex and zeroes otherwise.
#' 
#' @param complexes a list of complexes, with each entry containing complex
#' subunits as a character vector
#' 
#' @return an adjacency matrix between all complex subunits
#' 
#' @examples
#' data(gold_standard)
#' adj <- adjacency_matrix_from_list(gold_standard)
#' 
#' @importFrom purrr map_dfr
#' @importFrom tidyr crossing
#' 
#' @export
adjacency_matrix_from_list <- function(complexes) {
  proteins <- unique(unlist(complexes))
  n_proteins <- length(proteins)
  adjacency <- matrix(0, nrow = n_proteins, ncol = n_proteins,
                      dimnames = list(proteins, proteins))
  
  # identify pairwise interactions
  pairs <- map_dfr(complexes, ~ crossing(protein_A = ., protein_B = .))
  adjacency[as.matrix(pairs)] <- 1
  
  return(adjacency)
}

#' Preprocess a co-elution profile matrix
#' 
#' Clean a matrix of co-elution/co-fractionation profiles by 
#' (1) imputing single missing
#' values with the average of neighboring values, (2) replacing missing values
#' with random, near-zero noise, and (3) smoothing with a moving average
#' filter. 
#' 
#' @param profile_matrix a numeric matrix of co-elution profiles, with proteins
#' in rows, or a \code{\linkS4class{MSnSet}} object
#' @param impute_NA if true, impute single missing values with the average of
#' neighboring values 
#' @param smooth if true, smooth the chromatogram with a moving average filter
#' @param smooth_width width of the moving average filter, in fractions 
#' @param noise_floor mean value of the near-zero noise to add 
#' 
#' @return a cleaned matrix
#' 
#' @examples
#' data(scott)
#' mat <- scott[c(1, 16), ]
#' mat_clean <- clean_profiles(mat)
#' 
#' @importFrom MSnbase exprs
#' @importFrom Biobase exprs<-
#' @importFrom methods is
#' 
#' @export
clean_profiles <- function(profile_matrix, impute_NA = TRUE, smooth = TRUE, 
                           smooth_width = 4, noise_floor = 0.001) {
  if (is(profile_matrix, "MSnSet")) {
    profile_matrix <- exprs(profile_matrix)
  }
  
  profile_matrix <- t(apply(profile_matrix, 1, clean_profile, 
                            impute_NA = impute_NA,
                            smooth = smooth, 
                            smooth_width = smooth_width, 
                            noise_floor = noise_floor))
  
  return(profile_matrix)
}

#' Combine features across multiple replicates
#' 
#' Concatenate features extracted from multiple replicates to a single data 
#' frame that will be used as input to a classifier. Doing so allows the 
#' classifier to naturally weight evidence for an interaction between each
#' protein pair from each feature in each replicate in proportion to its
#' discriminatory power on known examples.
#' 
#' @param feature_list a list of feature data frames, as produced by 
#'   \code{\link[PrInCE]{calculate_features}}, with proteins in the first two 
#'   columns
#' 
#' @return a data frame containing features for all protein pairs across all 
#'   replicates 
#' 
#' @importFrom dplyr full_join
#' 
#' @export
concatenate_features <- function(feature_list) {
  features <- Reduce(function(x, y)
    full_join(x, y, by = colnames(x)[c(1, 2)]), 
    feature_list)
  return(features)
}

#' Filter a co-elution profile matrix
#' 
#' Filter a matrix of co-elution/co-fractionation profiles by removing
#' profiles without a certain number of non-mising or consecutive points.
#' 
#' @param profile_matrix a numeric matrix of co-elution profiles, with proteins
#'   in rows, or a \code{\linkS4class{MSnSet}} object
#' @param min_points filter profiles without at least this many total, 
#'   non-missing points
#' @param min_consecutive filter profiles without at least this many 
#'   consecutive, non-missing points
#' 
#' @return the filtered profile matrix
#' 
#' @examples
#' data(scott)
#' nrow(scott)
#' filtered <- filter_profiles(scott)
#' nrow(scott)
#' 
#' @importFrom MSnbase exprs
#' @importFrom Biobase exprs<-
#' @importFrom methods is
#' @importFrom purrr map_int
#' 
#' @export
filter_profiles <- function(profile_matrix, min_points = 1, 
                            min_consecutive = 5) {
  # filter profiles without N non-missing points
  nas <- is.na(profile_matrix)
  if (!is.na(min_points) & !is.null(min_points) & min_points > 0) {
    profile_matrix <- profile_matrix[rowSums(!nas) >= min_points, ]
  } else {
    # need to at least filter profiles without any points
    profile_matrix <- profile_matrix[rowSums(!nas) >= 1, ]
  }
  
  # filter profiles without N consecutive points after imputation
  if (!is.na(min_consecutive) & !is.null(min_consecutive) &
      min_consecutive > 0) {
    if (is(profile_matrix, "MSnSet")) {
      expr <- exprs(profile_matrix)
    } else {
      expr <- profile_matrix
    }
    imputed <- t(apply(expr, 1, impute_neighbors))
    imputed_nas <- is.na(imputed)
    rles <- apply(imputed_nas, 1, rle)
    max_consecutive <- map_int(rles, ~ max(.$lengths[.$values == FALSE]))
    profile_matrix <- profile_matrix[max_consecutive >= min_consecutive, ]
  }

  return(profile_matrix)
}

#' Fit a mixture of Gaussians to a chromatogram curve
#'
#' Fit mixtures of one or more Gaussians to the curve formed by a chromatogram
#' profile, using nonlinear least-squares. 
#' 
#' @param chromatogram a numeric vector corresponding to the chromatogram trace
#' @param n_gaussians the number of Gaussians to fit
#' @param max_iterations the number of times to try fitting the curve with
#' different initial conditions; defaults to 10
#' @param min_R_squared the minimum R-squared value to accept when fitting the
#' curve with different initial conditions; defaults to 0.5
#' @param method the method used to select the initial conditions for
#' nonlinear least squares optimization (one of "guess" or "random"); 
#' see \code{\link{make_initial_conditions}} for details
#' @param filter_gaussians_center true or false: filter Gaussians whose centres
#' fall outside the bounds of the chromatogram 
#' @param filter_gaussians_height Gaussians whose heights are below this 
#' fraction of the chromatogram height will be filtered. Setting this value to
#' zero disables height-based filtering of fit Gaussians
#' @param filter_gaussians_variance_min Gaussians whose variance falls below 
#' this number of fractions will be filtered. Setting this value to
#' zero disables filtering.
#' @param filter_gaussians_variance_max Gaussians whose variance is above
#' this number of fractions will be filtered. Setting this value to
#' zero disables filtering.
#'
#' @return a list with six entries: the number of Gaussians used to fit
#' the curve; the R^2 of the fit; the number of iterations used to 
#' fit the curve with different initial conditions; the coefficients of the 
#' fit model; and the fit curve predicted by the fit model.
#' 
#' @examples
#' data(scott)
#' chrom <- clean_profile(scott[1, ])
#' fit <- fit_gaussians(chrom, n_gaussians = 1)
#' 
#' @importFrom stats coef cor setNames nls
#' @importFrom dplyr first
#' 
#' @export
fit_gaussians <- function(chromatogram, n_gaussians,
                          max_iterations = 10, min_R_squared = 0.5,
                          method = c("guess", "random"),
                          filter_gaussians_center = TRUE,
                          filter_gaussians_height = 0.15,
                          filter_gaussians_variance_min = 0.1,
                          filter_gaussians_variance_max = 50) {
  indices <- seq_along(chromatogram)
  iter <- 0
  bestR2 <- 0
  bestCoefs <- NULL
  while (iter < max_iterations & bestR2 < min_R_squared) {
    # increment iteration counter
    iter <- iter + 1
    # make initial conditions
    initial_conditions <- make_initial_conditions(
      chromatogram, n_gaussians, method)
    A <- initial_conditions$A
    mu <- initial_conditions$mu
    sigma <- initial_conditions$sigma
    
    # fit the model
    p_model <- function(x, A, mu, sigma) {
      rowSums(sapply(seq_len(n_gaussians), 
                     function(i) A[i] * exp(-((x - mu[i])/sigma[i])^2)))
    }
    fit <- tryCatch({
      suppressWarnings(
        nls(chromatogram ~ p_model(indices, A, mu, sigma), 
            start = list(A = A, mu = mu, sigma = sigma), 
            trace = FALSE,  
            control = list(warnOnly = TRUE, minFactor = 1/2048)))
    }, error = function(e) { 
      e 
    }, simpleError = function(e) { 
      e
    })
    if ("error" %in% class(fit))
      next
    
    # split up fit coefficients vector into list with three entries
    coefs <- coef(fit)
    coefs <- split(coefs, rep(seq_len(3), each = n_gaussians))
    coefs <- setNames(coefs, c("A", "mu", "sigma"))
    
    # remove Gaussians with negative variances 
    if (filter_gaussians_variance_min > 0) {
      sigmas <- coefs[["sigma"]]
      drop <- which(sigmas < filter_gaussians_variance_min)
      if (length(drop) > 0)
        coefs <- lapply(coefs, `[`, -drop)
    }
    
    # remove Gaussians with extremely large 
    if (filter_gaussians_variance_max > 0) {
      sigmas <- coefs[["sigma"]]
      drop <- which(sigmas > filter_gaussians_variance_max)
      if (length(drop) > 0)
        coefs <- lapply(coefs, `[`, -drop)
    }
    
    # (optional): remove Gaussians outside bounds of chromatogram
    if (filter_gaussians_center) {
      means <- coefs[["mu"]]
      drop <- which(means < 0 | means > length(chromatogram))
      if (length(drop) > 0)
        coefs <- lapply(coefs, `[`, -drop)
    }
    
    # (optional): remove Gaussians less than 15% of height
    if (filter_gaussians_height > 0) {
      minHeight <- max(chromatogram) * filter_gaussians_height
      heights <- coefs[["A"]]
      drop <- which(heights < minHeight)
      if (length(drop) > 0)
        coefs <- lapply(coefs, `[`, -drop)
    }
    
    # calculate R^2
    if (length(first(coefs)) == 0)
      next
    curveFit <- fit_curve(coefs, indices)
    R2 <- cor(chromatogram, curveFit)^2
    # replace best fit with this model?
    if (R2 > bestR2 & R2 > min_R_squared) {
      bestR2 <- R2
      bestCoefs <- coefs
    }
  }
  if (!is.null(bestCoefs)) {
    curveFit <- fit_curve(bestCoefs, indices)
  } else {
    curveFit <- NULL
  }
  results <- list(n_gaussians = n_gaussians, R2 = bestR2, iterations = iter,
                  coefs = bestCoefs, curveFit = curveFit)
  return(results)
}

#' Predict interactions using an ensemble of classifiers 
#' 
#' Use an ensemble of classifiers to predict interactions from
#' co-elution dataset features. The ensemble approach ensures that 
#' results are robust to the partitioning of the dataset into folds. For each
#' model, the median of classifier scores across all folds is calculated.
#' Then, the median of all such medians across all models is calculated. 
#' 
#' @param dat a data frame containing interacting gene/protein pairs in the
#'   first two columns, and the features to use for classification in the 
#'   remaining columns
#' @param labels labels for each interaction in \code{dat}: 0 for negatives,
#'   1 for positives, and NA for interactions outside the reference set 
#' @param classifier the type of classifier to use; one of \code{"NB"} 
#'   (naive Bayes), \code{"SVM"} (support vector machine), \code{"RF"}
#'   (random forest), or \code{"LR"} (logistic regression)
#' @param models the number of classifiers to train
#' @param cv_folds the number of folds to split the reference dataset into 
#'   when training each classifier. By default, each 
#'   classifier uses ten-fold cross-validation, i.e., the classifier is trained
#'   on 90\% of the dataset and used to classify the remaining 10\%
#' @param trees for random forest classifiers only, the number of trees to 
#'   grow for each fold
#' @param node_columns a vector of length two, denoting either the indices 
#'   (integer vector) or column names (character vector) of the columns within 
#'   the input data frame containing the nodes participating in pairwise 
#'   interactions; defaults to the first two columns of the data frame 
#'   (\code{c(1, 2)})
#'  
#' @return the input data frame of pairwise interactions, ranked by the 
#' median of classifier scores across all ensembled models
#' 
#' @examples
#' ## calculate features
#' data(scott)
#' data(scott_gaussians)
#' subset <- scott[seq_len(500), ] ## limit to first 500 proteins
#' gauss <- scott_gaussians[names(scott_gaussians) %in% rownames(subset)]
#' features <- calculate_features(subset, gauss)
#' ## make training labels
#' data(gold_standard)
#' ref <- adjacency_matrix_from_list(gold_standard)
#' labels <- make_labels(ref, features)
#' ## predict interactions with naive Bayes classifier
#' ppi <- predict_ensemble(features, labels, classifier = "NB", 
#'                         cv_folds = 3, models = 1)
#' 
#' @importFrom stats predict binomial
#' @importFrom robustbase rowMedians
#' @importFrom progress progress_bar
#' @importFrom naivebayes naive_bayes
#' @importFrom LiblineaR LiblineaR
#' @importFrom ranger ranger
#' @importFrom speedglm speedglm
#' @importFrom dplyr arrange
#' 
#' @export
predict_ensemble <- function(dat, 
                             labels, 
                             classifier = c("NB", "SVM", "RF", "LR"), 
                             models = 1, 
                             cv_folds = 10,
                             trees = 500,
                             node_columns = c(1, 2)) {
  classifier <- match.arg(classifier)
  # length of node columns must be exactly two (pairwise interactions)
  if (length(node_columns) != 2) {
    stop("length of `node_columns` must be exactly 2")
  }
  
  ## define global variables to prevent check complaining
  score <- NULL
  
  # scale all features
  if (is.numeric(node_columns)) {
    node_colnames <- colnames(dat)[node_columns]
  } else if (is.character(node_columns)) {
    node_colnames <- node_columns
  } else {
    stop("`node_columns` must be an integer or character vector")
  }
  if (classifier == "SVM") {
    dat[, !colnames(dat) %in% node_colnames] <- sapply(
      dat[, !colnames(dat) %in% node_colnames], scale)
  }
  
  # extract training data
  training_idxs <- which(!is.na(labels))
  training_labels <- as.factor(labels[training_idxs])
  training <- dat[training_idxs, !colnames(dat) %in% node_colnames]
  
  # create matrix to hold medians from each model
  n_interactions <- nrow(dat)
  col1 <- node_columns[1]
  col2 <- node_columns[2]
  interaction_names <- paste0(dat[[col1]], "_", dat[[col2]])
  ensembled <- matrix(NA, ncol = models, nrow = n_interactions,
                      dimnames = list(interaction_names))
  
  # create progress bar
  total_models <- models * cv_folds
  pb <- progress_bar$new(
    format = "running fold :what [:bar] :percent eta: :eta",
    clear = FALSE, total = total_models, width = 80)
  counter <- 0
  
  # create models
  for (i in seq_len(models)) {
    folds <- cut(seq_len(nrow(training)), breaks = cv_folds, labels = FALSE)
    folds <- sample(folds) ## randomize
    clf_scores <- matrix(NA, ncol = cv_folds, nrow = n_interactions,
                         dimnames = list(interaction_names))
    for (fold in seq_len(cv_folds)) {
      # print message
      counter <- counter + 1
      pb$tick(tokens = list(what = sprintf(
        paste0("%-", nchar(total_models), "s"), counter)))
      
      # train model
      clf_data <- training[which(folds != fold),]
      clf_labels <- as.factor(training_labels[which(folds != fold)])
      clf_data_labeled <- cbind(clf_data, label = clf_labels)
      clf <- switch(classifier,
                    NB = naive_bayes(clf_data, clf_labels),
                    SVM = LiblineaR(clf_data, clf_labels, type = 2),
                    RF = ranger(data = clf_data_labeled, 
                                dependent.variable.name = "label",
                                probability = TRUE,
                                num.trees = trees,
                                num.threads = 1),
                    LR = speedglm(label ~ ., clf_data_labeled, 
                                  family = binomial()))
      
      # classify
      withheld_idxs <- as.integer(rownames(training))[folds == fold]
      test_data <- dat[-withheld_idxs, !colnames(dat) %in% node_colnames]
      predictions <- switch(
        classifier, 
        NB = predict(clf, test_data, type = 'prob', threshold = 5e-324),
        SVM = predict(clf, test_data, decisionValues = TRUE),
        RF = predict(clf, test_data, num.threads = 1),
        LR = predict(clf, test_data, type = 'response'))
      ## extract predictions as numeric vector
      predictions <- switch(
        classifier,
        NB = predictions[, "1"],
        SVM = -1.0 * predictions$decisionValues[, "0"],
        RF = predictions[[1]][, "1"],
        LR = predictions)
      # assign scores
      clf_scores[-withheld_idxs, fold] <- predictions
      
      # call GC
      gc()
    }
    medians <- setNames(rowMedians(clf_scores, na.rm = TRUE),
                        rownames(clf_scores))
    ensembled[, i] <- medians
  }
  
  # calculate median of medians across ensembled models
  ensembled_medians <- setNames(rowMedians(ensembled, na.rm = TRUE),
                                rownames(ensembled))
  
  # create ranked data frame
  interactions <- cbind(dat[, node_columns], score = ensembled_medians, 
                        label = labels)
  interactions <- arrange(interactions, -score)
  
  return(interactions)
}

#' Output the fit curve for a given mixture of Gaussians
#' 
#' For a Gaussian mixture model fit to a curve by \code{\link{fit_gaussians}},
#' output the fit curve using the coefficients rather than the \code{nls} 
#' object. This allows individual Gaussians to be removed from the fit model:
#' for example, if their height is below a certain threshold, or their 
#' centres are outside the bounds of the chromatogram. 
#' 
#' @param coef numeric vector of coefficients for a Gaussian mixture model fit 
#' by \code{\link{fit_gaussians}}. This function assumes that the heights of the
#' Gaussians are specified by coefficients beginning with "A" 
#' ("A1", "A2", "A3", etc.), centres are specified by coefficients beginning
#' with "mu", and standard deviations are specified by coefficients beginning
#' with "sigma".  
#' @param indices the indices, or x-values, to predict a fitted curve for 
#' (for example, the fractions in a given chromatogram)
#' 
#' @return the fitted curve
#' 
#' @examples
#' data(scott)
#' chrom <- clean_profile(scott[1, ])
#' fit <- fit_gaussians(chrom, n_gaussians = 1)
#' curve <- fit_curve(fit$coefs, seq_along(chrom))
#' 
#' @export
fit_curve <- function(coef, indices) {
  A <- coef[["A"]]
  mu <- coef[["mu"]]
  sigma <- coef[["sigma"]]
  gaussians <- length(A)
  rowSums(sapply(seq_len(gaussians), function(i) 
    A[i] * exp(-((indices - mu[i])/sigma[i])^2)))
}

#' Check the format of a list of Gaussians
#' 
#' Test whether an input list of Gaussians conforms to the format expected
#' by PrInCE: that is, a named list with five fields for each entry, i.e., the
#' number of Gaussians in the mixture model, the {\eqn{r^2}} value, the 
#' number of iterations used by \code{\link[stats]{nls}}, the coefficients of
#' each model, and the fitted curve. 
#' 
#' Optionally, some extra checks will be 
#' done on the fraction of proteins in the complete dataset for which a 
#' Gaussian mixture model could be fit, if provided. In particular, the function
#' will throw an error if fewer than \code{n_error} proteins have a fitted 
#' Gaussian, and emit a warning if fewer than \code{pct_warning} do. 
#'  
#' @param gaussians the list of Gaussians
#' @param proteins the complete set of input proteins
#' @param replicate_idx the replicate being analyzed, if input proteins are 
#'   provided; used to throw more informative error messages
#' @param n_error minimum number of proteins that can have fitted Gaussians
#'   without throwing an error
#' @param pct_warning minimum fraction of proteins that can have fitted
#'   Gaussians without giving a warning
#' 
#' @return \code{TRUE} if all conditions are met, but throws an error if any 
#'   is not
#'   
#' @examples 
#' data(scott_gaussians)
#' check_gaussians(scott_gaussians)
#' 
#' @importFrom purrr map_lgl
#' @importFrom dplyr first
#'   
#' @export
check_gaussians <- function(gaussians, proteins = NULL, replicate_idx = NULL,
                            n_error = 3, pct_warning = 0.1) {
  if (!missing(gaussians) && !is.null(gaussians)) {
    if (!is.list(gaussians)) {
      stop("`gaussians` object is not a list")
    }
    if (is.null(names(gaussians))) {
      stop("`gaussians` object is not a named list")
    }
    required_fields <- c("n_gaussians", "R2", "iterations", "coefs", "curveFit")
    check_fields <- map_lgl(gaussians, ~ length(
      setdiff(required_fields, names(.))) == 0)
    if (any(!check_fields)) {
      stop("not all gaussians have all required fields (", 
           paste0(required_fields, collapse = ", "), "), starting at index ",
           first(which(!check_fields)))
    }
  } else {
    stop("`gaussians` object is null")
  }
  if (!missing(proteins) && !is.null(proteins)) {
    repl_string <- ifelse(!is.null(replicate_idx),
                          paste(" in replicate ", replicate_idx), "")
    # error if too few proteins have Gaussians
    n_gaussians <- sum(proteins %in% names(gaussians))
    if (n_gaussians <= n_error) {
      stop("only ", n_gaussians, " proteins have fitted Gaussians", repl_string,
           " (minimum: ", n_error, ")")
    }
    # warn if a low % of proteins have Gaussians
    pct_gaussians <- mean(proteins %in% names(gaussians))
    if (pct_gaussians < pct_warning) {
      warning("only ", format(100 * pct_gaussians, digits = 2), 
              " of proteins have fitted Gaussians", repl_string, " (minimum: ", 
              format(100 * pct_warning, digits = 2), ")")
    }
  }
  return(TRUE)
}

#' Make labels for a classifier based on a gold standard
#' 
#' Create labels for a classifier for protein pairs in the same order as 
#' in a dataset that will be used as input to a classifier, in a 
#' memory-friendly way. 
#' 
#' @param gold_standard an adjacency matrix of gold-standard interactions
#' @param dat a data frame with interacting proteins in the first two
#'   columns 
#' @param node_columns a vector of length two, denoting either the indices 
#'   (integer vector) or column names (character vector) of the columns within 
#'   the data frame containing the nodes participating in pairwise interactions;
#'   defaults to the first two columns of the data frame (\code{c(1, 2)})
#' @param protein_groups optionally, specify a list linking each protein in the 
#'   first two columns of the input data frame to a protein group 
#' 
#' @return a vector of the same length as the input dataset, containing 
#' \code{NA}s for protein pairs not in the gold standard and ones or zeroes
#' based on the content of the adjacency matrix
#' 
#' @examples 
#' data(gold_standard)
#' adj <- adjacency_matrix_from_list(gold_standard)
#' proteins <- unique(unlist(gold_standard))
#' dat <- data.frame(protein_A = sample(proteins, 10), 
#'                   protein_B = sample(proteins, 10))
#' labels <- make_labels(adj, dat)
#' 
#' @importFrom purrr map map_lgl
#' @importFrom tidyr crossing
#' 
#' @export
make_labels <- function(gold_standard, dat, 
                        node_columns = c(1, 2),
                        protein_groups = NULL) {
  # length of node columns must be exactly two (pairwise interactions)
  if (length(node_columns) != 2) {
    stop("length of `node_columns` must be exactly 2")
  }
  col1 <- node_columns[1]
  col2 <- node_columns[2]
  proteins_1 <- as.character(dat[[col1]])
  proteins_2 <- as.character(dat[[col2]])
  if (is.null(protein_groups)) {
    lab_idxs <- proteins_1 %in% rownames(gold_standard) &
      proteins_2 %in% rownames(gold_standard)
    if (sum(lab_idxs) == 0)
      stop("no proteins overlap between input and gold standard")
    idxing_mat <- cbind(proteins_1[lab_idxs], proteins_2[lab_idxs])
    labels <- rep(NA, nrow(dat))
    labels[lab_idxs] <- gold_standard[idxing_mat]
  } else {
    # filter groups based on presence/absence in matrix
    protein_groups <- map(protein_groups, ~ .[. %in% rownames(gold_standard)])
    # identify rows that overlap with the gold standard matrix
    groups_1 <- protein_groups[proteins_1]
    groups_2 <- protein_groups[proteins_2]
    ref_proteins <- rownames(gold_standard)
    overlap_1 <- map_lgl(groups_1, ~ any(. %in% ref_proteins))
    overlap_2 <- map_lgl(groups_2, ~ any(. %in% ref_proteins))
    lab_idxs <- overlap_1 & overlap_2
    if (sum(lab_idxs) == 0)
      stop("no protein groups overlap between input and gold standard")
    # create labels
    labels <- rep(NA, nrow(dat))
    # map groups of length 1 by matrix indexing 
    lengths_1 <- lengths(groups_1)
    lengths_2 <- lengths(groups_2)
    single_lab_idxs <- lab_idxs & lengths_1 == 1 & lengths_2 == 1
    idxing_mat <- cbind(unlist(groups_1[single_lab_idxs]), 
                        unlist(groups_2[single_lab_idxs]))
    labels[single_lab_idxs] <- gold_standard[idxing_mat]
    # map groups of length > 1
    multi_lab_idxs <- lab_idxs & !single_lab_idxs
    labels[multi_lab_idxs] <- map_lgl(which(multi_lab_idxs), ~ any(
      gold_standard[as.matrix(crossing(groups_1[[.]], groups_2[[.]]))]))
  } 
  return(labels)
}

#' Calculate precision at each point in a sequence
#' 
#' Calculate the precision of a list of interactions at each point in the list,
#' given a set of labels.
#' 
#' @param labels a vector of zeroes (FPs) and ones (TPs)
#' 
#' @return a vector of the same length giving the precision at each point in the
#' input vector
#' 
#' @examples
#' ## calculate features
#' data(scott)
#' data(scott_gaussians)
#' subset <- scott[seq_len(500), ] ## limit to first 500 proteins
#' gauss <- scott_gaussians[names(scott_gaussians) %in% rownames(subset)]
#' features <- calculate_features(subset, gauss)
#' ## make training labels
#' data(gold_standard)
#' ref <- adjacency_matrix_from_list(gold_standard)
#' labels <- make_labels(ref, features)
#' ## predict interactions with naive Bayes classifier
#' ppi <- predict_ensemble(features, labels, classifier = "NB", cv_folds = 3, 
#'                         models = 1)
#' ## tag precision of each interaction
#' ppi$precision <- calculate_precision(ppi$label)
#' 
#' @export
calculate_precision <- function(labels) {
  if (is.factor(labels))
    labels <- as.integer(labels) - 1
  TPs <- cumsum(replace(labels, is.na(labels), 0))
  FPs <- cumsum(replace(labels == 0, is.na(labels), 0))
  precision <- TPs / (TPs + FPs)
}

#' Test whether a network is unweighted
#'  
#' @param network the network to analyze 
#' 
#' @return true if the input network is a square logical or numeric matrix
#' 
#' @examples 
#' data(gold_standard)
#' adj <- adjacency_matrix_from_list(gold_standard)
#' is_unweighted(adj) ## returns TRUE
#' 
#' @importFrom tester is_square_matrix is_logical_matrix is_numeric_matrix
#' 
#' @export
is_unweighted <- function(network) {
  # check input is square logical, integer, or numeric matrix
  square <- is_square_matrix(network)
  if (!square)
    stop("input is not a square matrix")
  lgl <- is_logical_matrix(network)
  dbl <- is_numeric_matrix(network)
  if (!lgl & !dbl)
    stop("input could not be converted to adjacency matrix")
  # convert to numeric 
  network <- network * 1
  # all values should be missing, zero, or one
  all(is.na(network) | network == 1 | network == 0)
}

#' Impute single missing values
#' 
#' Impute single missing values within a chromatogram profile as the average
#' of their neighbors.
#'
#' @param chromatogram a numeric vector corresponding to the chromatogram trace
#' 
#' @return the imputed chromatogram 
#' 
#' @examples
#' data(scott)
#' chrom <- scott[16, ]
#' imputed <- impute_neighbors(chrom)
#' 
#' @export
impute_neighbors <- function(chromatogram) {
  fractions <- length(chromatogram)
  nas <- which(is.na(chromatogram) | chromatogram == 0)
  for (i in nas) {
    prevIdx <- i - 1
    nextIdx <- i + 1
    if (prevIdx < 1 | nextIdx > fractions - 1)
      next
    neighbors <- c(chromatogram[prevIdx], chromatogram[nextIdx])
    if (!any(is.na(neighbors))) {
      chromatogram[i] <- mean(neighbors)
    }
  }
  
  return(chromatogram)
}

#! /bin/sh

PRODUCT="Prince"
PROGRAM="prince"
VERSION="20260505"
WEBSITE="https://www.princexml.com"

prefix=${1:-/usr/local}

base=$( dirname "$0" )

cd "$base" || exit 1

echo "$PRODUCT $VERSION"
echo
echo "Install directory"
echo "    This is the directory in which $PRODUCT $VERSION will be installed."
echo "    Press Enter to accept the default directory or enter an alternative."
printf "    [%s]: " "$prefix"

IFS= read -r input
if [ ! -z "$input" ] ; then
    prefix="$input"
fi

echo
echo "Installing $PRODUCT $VERSION..."

install_dir () {
    install -d "$1"
}

install_bin () {
    install -m 755 "$1" "$2"
}

install_data () {
    install -m 644 "$1" "$2"
}

# Create shell script

cat > prince <<EOF
#! /bin/sh

exec "$prefix/lib/$PROGRAM/bin/$PROGRAM" --prefix="$prefix/lib/$PROGRAM" "\$@"
EOF

# Test that we can create directories

install_dir "$prefix/bin" 2>/dev/null ||
{
echo "    Unable to create directories in $prefix"
echo "    (You may need to be logged in as root to install programs in system"
echo "    directories. Ask your system administrator, or try installing inside"
echo "    your home directory, such as $HOME/$PROGRAM-$VERSION)."
echo
exit 1
}

# Install shell script

install_bin prince "$prefix/bin" || exit 1

# Install everything else

echo "Creating directories..."

for dir in $( find "lib/$PROGRAM" -type d ) ; do

    install_dir "$prefix/$dir" || exit 1

done

echo "Installing files..."

for file in $( find "lib/$PROGRAM" -type f ) ; do

    dir=$( dirname "$file" )

    if [ -x "$file" ] ; then
	install_bin "$file" "$prefix/$dir" || exit 1
    else
	if [ "$file" = "lib/$PROGRAM/license/license.dat" ] ; then
	    if [ ! -f "$prefix/$file" ] ; then
		install_data "$file" "$prefix/$dir" || exit 1
	    fi
	else
	    install_data "$file" "$prefix/$dir" || exit 1
	fi
    fi

done

echo
echo "Installation complete."
echo "    Thank you for choosing $PRODUCT $VERSION, we hope you find it useful."
echo "    Please visit $WEBSITE for updates and development news."
echo

---
title: "Interactome reconstruction from co-elution data with PrInCE"
author:
- name: Michael Skinnider
  affiliation: Michael Smith Laboratories, University of British Columbia, 
    Vancouver, Canada
  email: michael.skinnider@msl.ubc.ca
package: PrInCE
abstract: >
  Co-elution proteomics refers to a family of high-throughput methods to map
  protein-protein interaction networks and their dynamics in cellular
  stimulation or differentiation.
  These methods, also referred to as co-migration, co-fractionation, or
  protein correlation profiling, involve separating interacting protein
  complexes on the basis of their diameter or biochemical properties.
  Protein-protein interactions can then be inferred for pairs of proteins
  with similar elution profiles.
  PrInCE implements a machine-learning approach to identify protein-protein
  interactions given a set of labelled examples, using features
  derived exclusively from the data.
  This allows PrInCE to infer high-quality protein interaction networks from
  raw proteomics data, without bias towards known interactions or functionally
  associated proteins, making PrInCE a unique computational resource for
  discovery.
  We provide an overview of the key functionalities of the PrInCE R package,
  and demonstrate an example of the analysis of data from a co-elution
  experiment investigating the response of the cytoplasmic interactome to
  Fas-mediated apoptosis.
output: 
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Interactome reconstruction from co-elution data with PrInCE}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
bibliography: bibliography.bib
csl: pnas.csl
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## Introduction: What is PrInCE?

Proteins are the central players of life at the molecular level.
Yet cellular functions are rarely accomplished by single proteins acting in 
isolation.
Instead, most biological processes are accomplished by the dynamic organization
of proteins and other biological macromolecules, such as RNA and DNA, into 
networks of physical interactions.
Systematic maps of these protein interaction networks can provide a 
"wiring diagram" to complement the "parts list" revealed by genome sequencing,
placing each protein into a functional context.
However, historically, protein interaction networks were mapped primarily 
using labour-intensive methods that involved tagging each protein for affinity 
purification, or heterologously expressing them in yeast.
Besides being labour intensive, these approaches also yielded static pictures 
of cellular networks that offered little insight into how these networks are 
rewired by stimulation or in differentiation.

Recently, a family of proteomic approaches, variously referred to as 
co-elution, co-migration, co-fractionation, or protein correlation profiling, 
has been developed that allow high-throughput mapping of protein interaction 
networks in native cellular conditions 
[@kristensen2012;@havugimana2012;@kirkwood2013].
A subset of these even enable investigators to identify dynamic rearrangements
in the protein-protein interactome in response to cellular stimulation 
[@kristensen2012;@scott2017], or across _in vivo_ samples, such as mouse 
tissues [@skinnider2018atlas].
The underlying principle that unifies different experimental protocols is to 
separate protein complexes into a number of fractions, on the basis of their 
size (diameter) or biochemical properties, and to perform quantitative 
proteomic analysis of the fractions.
Proteins with similar "profiles" across fractions can be inferred to 
physically interact. 
However, because the number of potential pairs grows quadratically with the 
number of proteins quantified, and the number of potential complexes grows 
even faster, specialized bioinformatic approaches are required to infer 
protein interaction networks from the raw proteomics data. 

PrInCE is an R package that uses a machine learning approach to infer 
protein-protein interaction networks at a user-defined level of precision 
from co-elution proteomics data.
The input to PrInCE consists of a matrix derived from a co-elution proteomics 
experiment, with quantitations for each protein in each fraction (PrInCE can 
also handle more than one such matrix, in the case of biological replicates). 
PrInCE also requires a set of 'gold standard' protein complexes to learn from. 
It then calculates a series of features for each possible protein pair; 
importantly, these are derived directly from the data, without incorporating 
any external knowledge, a step that minimizes bias towards the rediscovery of 
known interactions [@skinnider2018]. 
These features, and the accompanying gold standard, are used as input to the 
classifier, which learns to distinguish interacting and non-interacting pairs.
A cross-validation procedure is then used to score every potential protein 
pair in the dataset, which are then ranked by their score in descending order, 
and the precision (defined as the ratio of true positives to true positives 
plus false positives) is calculated at every point in this ranked list.
The user can then apply a precision threshold of their choice to this ranked 
list to infer the protein-protein interaction network from their experiment. 

## Example 1: Interactome rearrangements in apoptosis

To demonstrate the use of PrInCE, we will work through a small example that 
is derived from a subset of the data presented in Scott _et al._, 2017 
[@scott2017]. 
In this paper, the authors mapped rearrangements in the cytoplasmic and 
membrane interactome during Fas-mediated apoptosis.
Control and stimulated cytoplasmic and membrane interactomes were quantified 
in three replicates each, meaning the complete dataset consists of twelve 
replicates. 
In practice, each set of replicates would be analyzed together (for a total 
of four networks).
However, such a complete analysis of the dataset would take over an hour, so 
for this vignette we focus on a single replicate.
The replicate in question is the first cytoplasmic replicate from the 
Fas-stimulated condition, and is bundled with the PrInCE package; it can be 
loaded with the following command:

```{r warning=FALSE, message=FALSE}
library(PrInCE)
data(scott)
```

The dataset consists of ratiometric protein quantitations, achieved by SILAC 
(stable isotope labelling by amino acids in cell culture), for 1,560 proteins 
in 55 size exclusion chromatography (SEC) fractions:

```{r}
dim(scott)
```

Each protein was quantified in at least one fraction; however, many 
measurements are missing:

```{r}
scott[1:10, 1:5]
```

This scenario is common in co-elution data: for example, a protein will be 
absent entirely from a given SEC fraction if it does not form a complex with 
a molecular weight in the mass range of that fraction. 

To predict protein-protein interactions using PrInCE's machine-learning 
approach, we also need two additional pieces of information to train the 
classifier: a set of true positive interactions, and a set of true negative 
interactions.
In practice, we recommend providing a list of experimentally verified protein 
complexes: PrInCE assumes intra-complex interactions represent true positives, 
and inter-complex interactions represent true negatives.
These can be obtained from a number of sources, such as the CORUM database 
[@giurgiu2018], or our own previously reported custom subset of CORUM that 
removes complexes which may not remain intact under co-elution conditions
[@stacey2018].
In the PrInCE R package, we provide a third option which is distributed under 
a CC-BY license, consisting of a list of 477 human protein complexes 
from the Complex Portal [@meldal2018]. 

```{r}
data(gold_standard)
head(gold_standard)
```

### Predicting protein-protein interactions: one-step analysis

The main function of the PrInCE package, `PrInCE`, provides an end-to-end 
workflow for predicting protein-protein interaction networks from the 
raw co-elution data. 
Briefly, this function first filters proteins with too little information 
to permit data analysis, then cleans the profiles for the remaining proteins 
and fits a mixture of Gaussians to each cleaned profile. 
PrInCE then calculates six features for each protein pair, from either the raw 
profiles, the cleaned profiles, or the fitted Gaussian mixture models, and 
concatenates features across replicates if more than one replicate was used.
These features are used as input to a machine learning model, along with the 
set of 'gold standard' true positive and true negative interactions, which 
uses a ten-fold cross-validation procedure to assign scores to each protein 
pair.
Protein pairs are ranked by their classifier scores and the precision at each 
point in the ranked list is calculated.
The entire list is returned to a user, who can select a precision threshold 
that matches their needs.

Once we have loaded a co-elution matrix and list of gold standard protein 
complexes into R, inferring the protein-protein interaction network with 
PrInCE is therefore as simple as the following command:

```{r eval=FALSE}
# set the seed to ensure reproducible output
set.seed(0)
## not evaluated 
PrInCE(scott, gold_standard)
```

However, this command is not evaluated in order to provide some information 
on a further parameter that the `PrInCE` function takes. 
One of the six features that PrInCE uses to score protein-protein interactions 
is derived from fitting a mixture of Gaussians to each protein's elution 
profile. 
The process of Gaussian fitting also allows PrInCE to filter proteins with 
poor-quality elution profiles (i.e., proteins for which a Gaussian mixture 
could not be fit with an r^2^ value above some minimum, set to 0.5 by default).
However, the process of fitting Gaussian mixture models to thousands of 
curves is one of the more computationally intensive steps in PrInCE and 
consequently, the `PrInCE` function can also take a pre-computed list of 
fitted Gaussians, fit using the command `build_gaussians`:

```{r eval=FALSE}
# set the seed to ensure reproducible output
set.seed(0)
## not evaluated
build_gaussians(scott)
```

In practice, the ability to provide pre-computed Gaussians can also save 
time when trying different parameters in PrInCE, such as different types of 
classifiers (described in greater detail in the following section). 

We provide a list of fitted Gaussians for the `scott` dataset in the 
`scott_gaussians` object:

```{r}
data(scott_gaussians)
str(scott_gaussians[[3]])
```

We therefore run PrInCE using our precomputed Gaussian curves with the 
following command, allowing PrInCE to print information about the status of 
the analysis (`verbose = TRUE`) and limiting the number of cross-validation 
folds for the sake of time:

```{r}
# set the seed to ensure reproducible output
set.seed(0)
# one-step analysis
interactions <- PrInCE(scott, gold_standard,
                       gaussians = scott_gaussians, 
                       cv_folds = 3,
                       verbose = TRUE)
head(interactions, 50)
```

The columns in the output are as follows:

- `protein_A`: the identifier of the first protein in the pair;
- `protein_B`: the identifier of the second in the pair;
- `score`: the score assigned to the protein pair by the classifier
- `label`: if the protein pair is in the reference set, this value will be 
  `1` (for true positives) or `0` (for true negatives); for all other pairs, 
  the value is `NA`
- `precision`: the precision at this point in the ranked list 

Note that at the very top of the list, the precision is not defined if no 
true positives _and_ no true negatives have yet been encountered. 

In this toy example, the small size of our dataset and the small size of our 
gold-standard complexes mean that the precision curve is unstable below 
about 2,000 interactions:

```{r}
precision <- interactions$precision[1:10000]
plot(precision)
```

In most real examples, the precision curve shows a smoother decline. 

For illustrative purposes, we here threshold the network at 50% precision 
using the `threshold_precision` function:

```{r}
network <- threshold_precision(interactions, threshold = 0.5)
nrow(network)
```

This results in an unweighted protein-protein interaction network with 
`r nrow(network)` interactions.

### Predicting protein-protein interactions: step-by-step analysis

The `PrInCE` function accepts a large number of arguments that were 
omitted from the preceding discussion.
We have strived to set reasonable defaults for each of these parameters, 
based on analyses that have involved much of the human co-elution proteomics 
data in the public domain.
However, users may wish to change some of these defaults, based on the 
properties of their dataset or the biological questions motivating their 
investigation. 
Here, we provide an alternative workflow for analyzing the `scott` dataset 
in a step-by-step manner, and a discussion of some of the most important 
parameters. 

#### `build_gaussians`

The `build_gaussians` function in PrInCE can be broken down into three steps.
First, profiles are preprocessed by basic filtering and cleaning operations. 
Single missing values are imputed as the mean of their two neighboring points, 
and profiles with fewer than five consecutive points are filtered from 
further analysis.
Profiles are then cleaned by replacing missing values with near-zero noise, 
and smoothed with a moving average filter. 
Finally, mixtures of one to five Gaussians are fit to each profile using 
nonlinear least squares, and model selection is performed to retain the best 
mixture model for each curve.
Proteins that could not be fit with a mixture of Gaussians without an r^2^ 
value above some minimum are omitted. 

This function takes the following parameters:

- `min_consecutive`: the minimum number of consecutive points, after imputing 
  single missing values, to retain a profile; defaults to `5`
- `min_points`: the minimum number of total points to retain a profile; 
  defaults to `1` so that only the number of consecutive points is used to 
  filter profiles
- `impute_NA`: if `FALSE`, skip imputation of single missing values
- `smooth`: if `FALSE`, skip curve smoothing with the moving average filter
- `smooth_width`: width of the moving average filter, in fractions; 
  defaults to `4`
- `max_gaussians`: the maximum number of Gaussians with which to fit each 
  profile; defaults to `5`
- `criterion`: the criterion used for model selection; defaults to `AICc`, 
  the corrected Akaike information criterion; other options are `BIC` 
  (Bayesian information criterion) or `AIC` (Akaike information criterion)
- `max_iterations`: the maximum number of iterations to use for curve fitting 
  with random restarts
- `min_R_squared`: the minimum r^2^ value to retain the fitted curve; 
  defaults to `0.5`. Profiles that cannot be fit by a mixture of Gaussians are 
  assumed to be low-quality and excluded from further analysis by default.
- `method`: method used to select initial conditions for `nls`; can select 
  either random parameters (`random`) or make an educated guess based on the 
  maximum values in the profile (`guess`, the default)
- `filter_gaussians_center`, `filter_gaussians_height`, 
  `filter_gaussians_variance_min`, `filter_gaussians_variance_max`: 
  heuristics used to filter poor-quality Gaussian fits. If `TRUE` (default), 
  `filter_gaussians_center` will remove Gaussians whose mean falls outside 
  the bounds of the chromatogram. `filter_gaussians_height` controls the 
  minimum height of the Gaussians, while `filter_gaussians_variance_min` and 
  `filter_gaussians_variance_max` control the range of their standard deviation.  

All of these parameters except the last four are exposed through the `PrInCE`
function. 

As an example, we will re-analyze the `scott` dataset with stricter filtering 
criteria, requiring the presence of at least ten (non-imputed) data points 
in addition to five consecutive points; fitting with a maximum of three 
Gaussians, instead of five; and requiring a better fit than the default 
settings. 
For the sake of time, we allow only 10 iterations for the curve-fitting 
algorithm here, and we elect to fit only the first 500 profiles.

```{r}
data(scott)
# set the seed to ensure reproducible output
set.seed(0)
# fit Gaussians
gauss <- build_gaussians(scott[seq_len(500), ], 
                         min_points = 10, min_consecutive = 5, 
                         max_gaussians = 3, min_R_squared = 0.75,
                         max_iterations = 10)
# filter profiles that were not fit
scott <- scott[names(gauss), ]
```

By default, the profile matrix is filtered to exclude proteins whose elution 
profiles could not be fit by a mixture of Gaussians prior to featurization. 

#### `calculate_features` 

Having fitted our co-elution profiles with Gaussians and filtered them 
accordingly, the next step is to calculate features for each protein pair.
This is done using the `calculate_features` function.
By default, PrInCE calculates six features from each pair of co-elution 
profiles as input to the classifier, including conventional similarity metrics 
but also several features specifically adapted to co-elution proteomics. 
The complete set of features includes:

1. the Pearson correlation between raw co-elution profiles;
2. the p-value of the Pearson correlation between raw co-elution profiles;
3. the Pearson correlation between cleaned profiles, which are generated by 
   imputing single missing values with the mean of their neighbors, replacing 
   remaining missing values with random near-zero noise, and smoothing the 
   profiles using a moving average filter (see `clean_profile`);
4. the Euclidean distance between cleaned profiles;
5. the 'co-peak' score, defined as the distance, in fractions, between the 
   maximum values of each profile; and
6. the 'co-apex' score, defined as the minimum Euclidean distance between any 
   pair of fit Gaussians

In addition to the profile matrix and list of fitted Gaussian mixtures, the 
`calculate_features` function takes six parameters that enable the user to 
enable or disable each of these six features (in order, `pearson_R_raw`, 
`pearson_P`, `pearson_R_cleaned`, `euclidean_distance`, `co_peak`, and 
`co_apex`). 
By default, all six are enabled.

Continuing our example, if we wanted the classifier to omit the Euclidean 
distance, we could disable this feature using the following command: 

```{r}
feat <- calculate_features(scott, gauss, euclidean_distance = FALSE)
head(feat)
```

If we had multiple replicates, we would here concatenate them into a single 
feature data frame using the `concatenate_features` function:

```{r eval=FALSE}
## not run
# concatenate features from three different `scott` replicates
feat1 <- calculate_features(scott1, gauss1)
feat2 <- calculate_features(scott2, gauss2)
feat3 <- calculate_features(scott3, gauss3)
feat <- concatenate_features(list(feat1, feat2, feat3))
```

#### `predict_interactions`

With our features in hand and a list of gold standard protein complexes, we can 
now provide these to a machine-learning classifier to rank potential 
interactions.
This is accomplished using the `predict_interactions` function.
In order to score interactions that are also part of the reference set, PrInCE
uses a cross-validation strategy, randomly splitting the reference data into 
ten folds, and using each split to score interactions in one of the folds 
without including them in the training data.
For interactions that are not part of the training set, the median score over 
all ten folds is returned.
In addition, to ensure that the results are not sensitive to the way in which 
the dataset is split, PrInCE averages predictions over an ensemble of ten 
classifiers, each with different cross-validation splits. 
By default, PrInCE uses a naive Bayes classifier.
However, the PrInCE R package also implements three other types of classifiers: 
support vector machine, random forest, and logistic regression.
In addition, PrInCE offers an option to ensemble results over multiple 
different classifiers (sometimes called "heterogeneous classifier fusion" 
[@riniker2013]). 
In this option, cross-validation and ensembling is performed for all four 
types of classifiers independently, then the ranks of each protein pair 
across all four classifiers are averaged to return the final ranked list. 

These options are controlled using the following parameters:

- `classifier`: the type of classifier to use; one of `NB`, `SVM`, `RF`, `LR`, 
  or `ensemble`, corresponding to the options described above
- `models`: the size of the ensemble for each classifier type, i.e., the 
  number of models to train, each with a different train-test split
- `cv_folds`: the number of folds to use in k-fold cross-validation
- `trees`: for random forest and heterogeneous classifier fusion only, 
  the number of trees in each RF model

Continuing our example, we will demonstrate the use of a support vector 
machine to rank potential interactions (`classifier = "SVM"`). 
For the sake of time, we use a single model (omitting ensembling; `models = 1`)
and only three-fold cross-validation folds (`cv_folds = 3`).
To use our list of protein complexes as a gold standard, we must first convert
it to an adjacency matrix; this is done using the helper function 
`adjacency_matrix_from_list` (see also the related function 
`adjacency_matrix_from_data_frame`). 

```{r}
data(gold_standard)
reference <- adjacency_matrix_from_list(gold_standard)
# set the seed to ensure reproducible output
set.seed(0)
# predict interactions
ppi <- predict_interactions(feat, reference, classifier = "SVM",
                            models = 1, cv_folds = 3)
```

We can now plot the precision curve over the first 20,000 interactions:

```{r}
precision <- ppi$precision[seq_len(2e4)]
plot(precision)
```

Finally, we will likely want to keep only the set of high-confidence
interactions for further analysis, where "confidence" is quantified using 
precision. 
This is accomplished using the `threshold_precision` function.
For example, the following command constructs a protein-protein interaction 
network at 70% precision: 

```{r}
net <- threshold_precision(ppi, threshold = 0.7)
nrow(net)
```

### Identifying co-eluting protein complexes

The core functionality of PrInCE involves the use of a machine-learning 
framework to predict binary interactions from co-elution data, with discovery
of novel interactions being a primary goal.
However, PrInCE also implements one alternative to this analytical framework, 
which asks whether statistically significant co-elution is observed for 
_known_ protein complexes.  
This is achieved using a permutation-based approach, inspired by methods 
developed for another proteomic method for interactome profiling,
thermal proximity co-aggregation (TPCA) [@tan2018]. 
Briefly, given a list of known complexes, PrInCE calculates the median 
Pearson correlation between all pairs of complex members.
(To reduce the effect of spurious correlations between proteins that are 
rarely observed in the same fractions, PrInCE requires a certain minimum
number of paired observations to include any given correlation in this 
analysis---by default, 10 pairs).
Then, PrInCE simulates a large number of complexes of equivalent size 
(by default, 100), and calculates the median Pearson correlation between
pairs of random 'complexes'. 
The resulting null distribution is used to assess the statistical significance
of the observed co-elution profile at the protein complex level. 

To identify complexes from the Complex Portal dataset that are significantly
co-eluting in this replicate, we first use PrInCE's `filter_profiles` and
`clean_profiles` functions:

```{r}
# analyze cleaned profiles
data(scott)
filtered = filter_profiles(scott)
chromatograms = clean_profiles(filtered)
```

The `filter_profiles` function uses a permissive set of filters to discard
chromatograms that do not contain enough information to make inferences
about that protein's interaction partners. 
Similarly, the `clean_profiles` applies some simple preprocessing steps to the
filtered chromatograms.
By default, this function is applied to calculate Pearson correlations
during interaction prediction in PrInCE. 
It imputes single missing values as the average of the two neighbors,
remaining missing values with near-zero noise, then passes a moving-average
filter over the chromatogram to smooth it. 

We can now test for complex co-elution in the preprocessed chromatogram matrix
using the `detect_complexes` function:

```{r}
# detect significantly co-eluting complexes
set.seed(0)
z_scores = detect_complexes(chromatograms, gold_standard)
```

Complexes that could not be tested (that is, with fewer than three complex 
members present in the elution matrix) are given `NA` values, which we remove.

```{r}
# remove complexes that could not be analyzed
z_scores = na.omit(z_scores)
# how many could be tested?
length(z_scores)
# how many were significant at uncorrected, two-tailed p < 0.05?
sum(z_scores > 1.96)
# print the top complexes
head(sort(z_scores, decreasing = TRUE))
```

Of the 23 complexes that could be tested in this (unusually sparse) replicate, 
13 were significant at an uncorrected, two-tailed p-value threshold of 0.05 

## Example 2: Interactome of HeLa cells

As a second example, we can reanalyze another dataset bundled with the PrInCE
R package.
This dataset consists of a subset of the data presented by Kristensen _et al._, 
2012 [@kristensen2012], who applied SEC-PCP-SILAC to monitor the interactome 
of HeLa cell lysates, then mapped interactome rearrangements induced by 
epidermal growth factor (EGF) stimulation. 
Three biological replicate experiments were performed, and in practice, 
all three replicates from each condition would be analyzed together.
However, for the purposes of demonstrating the use of the PrInCE R package, 
we limit our analysis to the first replicate from the unstimulated condition. 

We first load the data matrix and fitted Gaussians, provided with the PrInCE
R package:

```{r}
data("kristensen")
data("kristensen_gaussians")
dim(kristensen)
length(kristensen_gaussians)
```

The co-elution matrix contains quantifications for 1,875 proteins across 48
SEC fractions.
Mixtures of Gaussians were fit to 1,117 of these. 
For the sake of time, we subset this matrix further to the first 500 proteins:

```{r}
kristensen <- kristensen[names(kristensen_gaussians), ]
kristensen <- kristensen[seq_len(500), ]
kristensen_gaussians <- kristensen_gaussians[rownames(kristensen)]
```

We also have to load our reference set of binary interactions or protein 
complexes, which in this case is derived from the Complex Portal human 
complexes.

```{r}
data("gold_standard")
head(gold_standard, 5)
```

We can predict interactions in a single step using the main `PrInCE` function, 
here using a single model (instead of the default ensemble of ten) and five
cross-validation folds (instead of the default of ten) for time: 

```{r}
# set seed for reproducibility
set.seed(0)
# predict interactions
interactions <- PrInCE(profiles = kristensen, 
                       gold_standard = gold_standard,
                       gaussians = kristensen_gaussians,
                       models = 1,
                       cv_folds = 5)
```

Finally, we can subset our list of interactions to obtain set of high-confidence
interactions for further analysis, using a relaxed precision cutoff of 50%. 

```{r}
network <- threshold_precision(interactions, 0.5)
nrow(network)
```

PrInCE predicts a total of 1,047 interactions at a precision of 50%. 

## Session info

```{r}
sessionInfo()
```

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_initial_conditions.R
\name{make_initial_conditions}
\alias{make_initial_conditions}
\title{Make initial conditions for curve fitting with a mixture of Gaussians}
\usage{
make_initial_conditions(
  chromatogram,
  n_gaussians,
  method = c("guess", "random"),
  sigma_default = 2,
  sigma_noise = 0.5,
  mu_noise = 1.5,
  A_noise = 0.5
)
}
\arguments{
\item{chromatogram}{a numeric vector corresponding to the chromatogram trace}

\item{n_gaussians}{the number of Gaussians being fit}

\item{method}{one of "guess" or "random", discussed above}

\item{sigma_default}{the default mean initial value of sigma}

\item{sigma_noise}{the amount of random noise to add or subtract from 
the default mean initial value of sigma}

\item{mu_noise}{the amount of random noise to add or subtract from the 
Gaussian centers in "guess" mode}

\item{A_noise}{the amount of random noise to add or subtract from the 
Gaussian heights in "guess" mode}
}
\value{
a list of three numeric vectors (A, mu, and sigma), each having
a length equal to the maximum number of Gaussians to fit
}
\description{
Construct a set of initial conditions for curve fitting using nonlinear
least squares using a mixture of Gaussians. The "guess" method ports 
code from the Matlab release of PrInCE. This method finds local maxima 
within the chromatogram, orders them by their separation (in number of 
fractions) from the previous local maxima, and uses the positions and heights
of these local maxima (+/- some random noise) as initial conditions for 
Gaussian curve-fitting. The "random" method simply picks random values 
within the fraction and intensity intervals as starting points for Gaussian
curve-fitting. The initial value of sigma is set by default to a random 
number within +/- 0.5 of two for both modes; this is based on our manual 
inspection of a large number of chromatograms.
}
\examples{
data(scott)
chrom <- clean_profile(scott[16, ])
set.seed(0)
start <- make_initial_conditions(chrom, n_gaussians = 2, method = "guess")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_gaussians.R
\name{build_gaussians}
\alias{build_gaussians}
\title{Deconvolve profiles into Gaussian mixture models}
\usage{
build_gaussians(
  profile_matrix,
  min_points = 1,
  min_consecutive = 5,
  impute_NA = TRUE,
  smooth = TRUE,
  smooth_width = 4,
  max_gaussians = 5,
  criterion = c("AICc", "AIC", "BIC"),
  max_iterations = 50,
  min_R_squared = 0.5,
  method = c("guess", "random"),
  filter_gaussians_center = TRUE,
  filter_gaussians_height = 0.15,
  filter_gaussians_variance_min = 0.5,
  filter_gaussians_variance_max = 50
)
}
\arguments{
\item{profile_matrix}{a numeric matrix of co-elution profiles, with proteins
in rows, or a \code{\linkS4class{MSnSet}} object}

\item{min_points}{filter profiles without at least this many total, 
non-missing points; passed to \code{\link{filter_profiles}}}

\item{min_consecutive}{filter profiles without at least this many 
consecutive, non-missing points; passed to \code{\link{filter_profiles}}}

\item{impute_NA}{if true, impute single missing values with the average of
neighboring values; passed to \code{\link{clean_profiles}}}

\item{smooth}{if true, smooth the chromatogram with a moving average filter;
passed to \code{\link{clean_profiles}}}

\item{smooth_width}{width of the moving average filter, in fractions;
passed to \code{\link{clean_profiles}}}

\item{max_gaussians}{the maximum number of Gaussians to fit; defaults to 5.
Note that Gaussian mixtures with more parameters than observed (i.e., 
non-zero or NA) points will not be fit. Passed to  
\code{\link{choose_gaussians}}}

\item{criterion}{the criterion to use for model selection;
one of "AICc" (corrected AIC, and default), "AIC", or "BIC". Passed to
\code{\link{choose_gaussians}}}

\item{max_iterations}{the number of times to try fitting the curve with
different initial conditions; defaults to 50. Passed to 
\code{\link{fit_gaussians}}}

\item{min_R_squared}{the minimum R-squared value to accept when fitting the
curve with different initial conditions; defaults to 0.5. Passed to 
\code{\link{fit_gaussians}}}

\item{method}{the method used to select the initial conditions for
nonlinear least squares optimization (one of "guess" or "random"); 
see \code{\link{make_initial_conditions}} for details. Passed to 
\code{\link{fit_gaussians}}}

\item{filter_gaussians_center}{true or false: filter Gaussians whose centres
fall outside the bounds of the chromatogram. Passed to 
\code{\link{fit_gaussians}}}

\item{filter_gaussians_height}{Gaussians whose heights are below this 
fraction of the chromatogram height will be filtered. Setting this value to
zero disables height-based filtering of fit Gaussians. Passed to 
\code{\link{fit_gaussians}}}

\item{filter_gaussians_variance_min}{Gaussians whose variance falls below 
this number of fractions will be filtered. Setting this value to
zero disables filtering. Passed to 
\code{\link{fit_gaussians}}}

\item{filter_gaussians_variance_max}{Gaussians whose variance is above
this number of fractions will be filtered. Setting this value to
zero disables filtering. Passed to 
\code{\link{fit_gaussians}}}
}
\value{
a list of fit Gaussian mixture models, where each item in the 
list contains the following five fields: the number of Gaussians used to fit
the curve; the R^2 of the fit; the number of iterations used to 
fit the curve with different initial conditions; the coefficients of the 
fit model; and the curve predicted by the fit model. Profiles that 
could not be fit by a Gaussian mixture model above the minimum R-squared 
cutoff will be absent from the returned list.
}
\description{
Identify peaks in co-fractionation profiles by deconvolving peaks in 
Gaussian mixture models. Models are mixtures of between 1 and 5 Gaussians.
Profiles are pre-processed prior to building Gaussians by filtering and 
cleaning. By default, profiles with fewer than 5 non-missing points, or 
fewer than 5 consecutive points after imputation of single missing values,
are removed. Profiles are cleaned by replacing missing values with 
near-zero noise, imputing single missing values as the mean of neighboring 
points, and smoothing with a moving average filter.
}
\examples{
data(scott)
mat <- clean_profiles(scott[seq_len(5), ])
gauss <- build_gaussians(mat, max_gaussians = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjacency_matrix_from_list.R
\name{adjacency_matrix_from_list}
\alias{adjacency_matrix_from_list}
\title{Create an adjacency matrix from a list of complexes}
\usage{
adjacency_matrix_from_list(complexes)
}
\arguments{
\item{complexes}{a list of complexes, with each entry containing complex
subunits as a character vector}
}
\value{
an adjacency matrix between all complex subunits
}
\description{
Convert a list of complexes into a pairwise adjacency matrix. The resulting
square adjacency matrix contains ones for proteins that are found in the
same complex and zeroes otherwise.
}
\examples{
data(gold_standard)
adj <- adjacency_matrix_from_list(gold_standard)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gaussian_aic.R
\name{aic}
\alias{aic}
\alias{gaussian_aic}
\alias{gaussian_aicc}
\alias{gaussian_bic}
\title{Model selection for Gaussian mixture models}
\usage{
gaussian_aic(coefs, chromatogram)

gaussian_aicc(coefs, chromatogram)

gaussian_bic(coefs, chromatogram)
}
\arguments{
\item{coefs}{the coefficients of the Gaussian mixture model, output by 
\code{\link{fit_gaussians}}}

\item{chromatogram}{the raw elution profile}
}
\value{
the AIC, corrected AIC, or BIC of the fit model
}
\description{
Calculate the AIC, corrected AIC, or BIC for a curve fit with a Gaussian 
mixture model by nonlinear least squares optimization. This function 
permits the calculation of the AIC/AICc/BIC after rejecting some Gaussians
in the model, for example because their centres are outside the bounds of 
the profile.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrInCE.R
\name{PrInCE}
\alias{PrInCE}
\title{PrInCE: Prediction of Interactomes from Co-Elution}
\usage{
PrInCE(
  profiles,
  gold_standard,
  gaussians = NULL,
  precision = NULL,
  verbose = FALSE,
  min_points = 1,
  min_consecutive = 5,
  min_pairs = 3,
  impute_NA = TRUE,
  smooth = TRUE,
  smooth_width = 4,
  max_gaussians = 5,
  max_iterations = 50,
  min_R_squared = 0.5,
  method = c("guess", "random"),
  criterion = c("AICc", "AIC", "BIC"),
  pearson_R_raw = TRUE,
  pearson_R_cleaned = TRUE,
  pearson_P = TRUE,
  euclidean_distance = TRUE,
  co_peak = TRUE,
  co_apex = TRUE,
  n_pairs = FALSE,
  classifier = c("NB", "SVM", "RF", "LR", "ensemble"),
  models = 1,
  cv_folds = 10,
  trees = 500
)
}
\arguments{
\item{profiles}{the co-elution profile matrix, or a list of profile matrices
if replicate experiments were performed. Can be a single numeric matrix,
with proteins in rows and fractions in columns, or a list of matrices.
Alternatively, can be provided as a single
\code{\linkS4class{MSnSet}} object or a list of objects.}

\item{gold_standard}{a set of 'gold standard' interactions, used to train the
classifier. Can be provided either as an adjacency matrix, in which
both rows and columns correspond to protein IDs in the co-elution matrix
or matrices, or as a list of proteins in the same complex, which will be
converted to an adjacency matrix by PrInCE. Zeroes in the adjacency matrix
are interpreted by PrInCE as "true negatives" when calculating precision.}

\item{gaussians}{optionally, provide Gaussian mixture models fit by
the \code{\link[PrInCE]{build_gaussians}} function. If \code{profiles} is
a numeric matrix, this should be the named list output by
\code{\link[PrInCE]{build_gaussians}} for that matrix; if \code{profiles}
is a list of numeric matrices, this should be a list of named lists}

\item{precision}{optionally, return only interactions above the given
precision; by default, all interactions are returned and the user can
subsequently threshold the list using the
\code{\link[PrInCE]{threshold_precision}} function}

\item{verbose}{if \code{TRUE}, print a series of messages about the stage
of the analysis}

\item{min_points}{filter profiles without at least this many total,
non-missing points; passed to \code{\link{filter_profiles}}}

\item{min_consecutive}{filter profiles without at least this many
consecutive, non-missing points; passed to \code{\link{filter_profiles}}}

\item{min_pairs}{minimum number of overlapping fractions between any given
protein pair to consider a potential interaction}

\item{impute_NA}{if true, impute single missing values with the average of
neighboring values; passed to \code{\link{clean_profiles}}}

\item{smooth}{if true, smooth the chromatogram with a moving average filter;
passed to \code{\link{clean_profiles}}}

\item{smooth_width}{width of the moving average filter, in fractions;
passed to \code{\link{clean_profiles}}}

\item{max_gaussians}{the maximum number of Gaussians to fit; defaults to 5.
Note that Gaussian mixtures with more parameters than observed (i.e.,
non-zero or NA) points will not be fit. Passed to
\code{\link{choose_gaussians}}}

\item{max_iterations}{the number of times to try fitting the curve with
different initial conditions; defaults to 50. Passed to
\code{\link{fit_gaussians}}}

\item{min_R_squared}{the minimum R-squared value to accept when fitting the
curve with different initial conditions; defaults to 0.5. Passed to
\code{\link{fit_gaussians}}}

\item{method}{the method used to select the initial conditions for
nonlinear least squares optimization (one of "guess" or "random");
see \code{\link{make_initial_conditions}} for details. Passed to
\code{\link{fit_gaussians}}}

\item{criterion}{the criterion to use for model selection;
one of "AICc" (corrected AIC, and default), "AIC", or "BIC". Passed to
\code{\link{choose_gaussians}}}

\item{pearson_R_raw}{if true, include the Pearson correlation (R) between
raw profiles as a feature}

\item{pearson_R_cleaned}{if true, include the Pearson correlation (R) between
cleaned profiles as a feature}

\item{pearson_P}{if true, include the P-value of the Pearson correlation
between raw profiles as a feature}

\item{euclidean_distance}{if true, include the Euclidean distance between
cleaned profiles as a feature}

\item{co_peak}{if true, include the 'co-peak score' (that is, the distance,
in fractions, between the single highest value of each profile) as a
feature}

\item{co_apex}{if true, include the 'co-apex score' (that is, the minimum
Euclidean distance between any pair of fit Gaussians) as a feature}

\item{n_pairs}{if \code{TRUE}, include the number of fractions in which
both of a given pair of proteins were detected as a feature}

\item{classifier}{the type of classifier to use: one of \code{"NB"} (naive
Bayes), \code{"SVM"} (support vector machine), \code{"RF"} (random forest),
\code{"LR"} (logistic regression), or \code{"ensemble"} (an ensemble of
all four)}

\item{models}{the number of classifiers to train and average across, each
with a different k-fold cross-validation split}

\item{cv_folds}{the number of folds to use for k-fold cross-validation}

\item{trees}{for random forests only, the number of trees in the forest}
}
\value{
a ranked data frame of interacting proteins, with the precision
  at each point in the list
}
\description{
PrInCE is a computational approach to infer protein-protein interaction
networks from co-elution proteomics data, also called co-migration,
co-fractionation, or protein correlation profiling. This family of methods
separates interacting protein complexes on the basis of their diameter
or biochemical properties. Protein-protein interactions can then be
inferred for pairs of proteins with similar elution profiles. PrInCE
implements a machine-learning approach to identify protein-protein
interactions given a set of labelled examples, using features derived
exclusively from the data. This allows PrInCE to infer high-quality
protein interaction networks from raw proteomics data, without bias
towards known interactions or functionally associated proteins, making
PrInCE a unique resource for discovery.
}
\details{
PrInCE takes as input a co-elution matrix, with detected proteins in rows and
fractions as columns, and a set of 'gold standard' true positives and true
negatives. If replicate experiments were performed, a list of co-elution
matrices can be provided as input. PrInCE will construct features for each
replicate separately and use features from all replicates as input to the
classifier. The 'gold standard' can be either a data frame or adjacency
matrix of known interactions (and non-interactions), or a list of protein
complexes. For computational convenience, Gaussian mixture models can be
pre-fit to every profile and provided separately to the \code{PrInCE}
function. The matrix, or matrices, can be provided to PrInCE either as
numeric matrices or as \code{\linkS4class{MSnSet}} objects.

PrInCE implements three different types of classifiers to predict
protein-protein interaction networks, including naive Bayes (the default),
random forests, and support vector machines. The classifiers are trained
on the gold standards using a ten-fold cross-validation procedure, training
on 90% of the data and predicting on the remaining 10%. For protein pairs
that are part of the training data, the held-out split is used to assign
a classifier score, whereas for the remaining protein pairs, the median of
all ten folds is used. Furthermore, to ensure the results are not sensitive
to the precise classifier split used, an ensemble of multiple classifiers
(ten, by default) is trained, and the classifier score is subsequently
averaged across classifiers. PrInCE can also ensemble across a set of
classifiers.

By default, PrInCE calculates six features from each pair of co-elution
profiles as input to the classifier, including conventional similarity
metrics but also several features specifically adapted to co-elution
proteomics. For example, one such feature is derived
from fitting a Gaussian mixture model to each elution profile, then
calculating the smallest Euclidean distance between any pair of fitted
Gaussians. The complete set of features includes:

\enumerate{
  \item the Pearson correlation between raw co-elution profiles;
  \item the p-value of the Pearson correlation between raw co-elution
    profiles;
  \item the Pearson correlation between cleaned profiles, which are generated
    by imputing single missing values with the mean of their neighbors,
    replacing remaining missing values with random near-zero noise, and
    smoothing the profiles using a moving average filter (see
    \code{\link[PrInCE]{clean_profile}});
 \item the Euclidean distance between cleaned profiles;
 \item the 'co-peak' score, defined as the distance, in fractions, between
   the maximum values of each profile; and
 \item the 'co-apex' score, defined as the minimum Euclidean distance between
   any pair of fit Gaussians
}

The output of PrInCE is a ranked data frame, containing the classifier score
for every possible protein pair. PrInCE also calculates the precision at
every point in this ranked list, using the 'gold standard' set of protein
complexes or binary interactions. Our recommendation is to select a
threshold for the precision and use this to construct an unweighted
protein interaction network.
}
\examples{
data(scott)
data(scott_gaussians)
data(gold_standard)
# analyze only the first 100 profiles
subset <- scott[seq_len(500), ]
gauss <- scott_gaussians[names(scott_gaussians) \%in\% rownames(subset)]
ppi <- PrInCE(subset, gold_standard,
  gaussians = gauss, models = 1,
  cv_folds = 3
)
}
\references{
\insertRef{stacey2017}{PrInCE}

\insertRef{scott2015}{PrInCE}

\insertRef{kristensen2012}{PrInCE}

\insertRef{skinnider2018}{PrInCE}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replace_missing_data.R
\name{replace_missing_data}
\alias{replace_missing_data}
\title{Replace missing data with median ± random noise}
\usage{
replace_missing_data(dat, noise_pct = 0.05)
}
\arguments{
\item{dat}{the data frame to replace missing data in}

\item{noise_pct}{the standard deviation of the random normal 
distribution from which to draw added noise, expressed as a 
percentage of the standard deviation of the non-missing values in each 
column}
}
\value{
a data frame with missing values in each numeric column replaced
by the column median, plus or minus some random noise
}
\description{
Replace missing data within each numeric column of a data frame with 
the column median, plus or minus some random noise, in order to train 
classifiers that do not easily ignore missing data (e.g. random forests or
support vector machines).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_features.R
\name{calculate_features}
\alias{calculate_features}
\title{Calculate the default features used to predict interactions in PrInCE}
\usage{
calculate_features(
  profile_matrix,
  gaussians,
  min_pairs = 0,
  pearson_R_raw = TRUE,
  pearson_R_cleaned = TRUE,
  pearson_P = TRUE,
  euclidean_distance = TRUE,
  co_peak = TRUE,
  co_apex = TRUE,
  n_pairs = FALSE,
  max_euclidean_quantile = 0.9
)
}
\arguments{
\item{profile_matrix}{a numeric matrix of co-elution profiles, with proteins
in rows, or a \code{\linkS4class{MSnSet}} object}

\item{gaussians}{a list of Gaussian mixture models fit to the profile matrix
by \code{link{build_gaussians}}}

\item{min_pairs}{minimum number of overlapping fractions between any given
protein pair to consider a potential interaction}

\item{pearson_R_raw}{if true, include the Pearson correlation (R) between
raw profiles as a feature}

\item{pearson_R_cleaned}{if true, include the Pearson correlation (R) between
cleaned profiles as a feature}

\item{pearson_P}{if true, include the P-value of the Pearson correlation 
between raw profiles as a feature}

\item{euclidean_distance}{if true, include the Euclidean distance between
cleaned profiles as a feature}

\item{co_peak}{if true, include the 'co-peak score' (that is, the distance,
in fractions, between the single highest value of each profile) as a feature}

\item{co_apex}{if true, include the 'co-apex score' (that is, the minimum
Euclidean distance between any pair of fit Gaussians) as a feature}

\item{max_euclidean_quantile}{very high Euclidean distance values are trimmed
to avoid numerical precision issues; values above this quantile will be
replaced with the value at this quantile (default: \code{0.9})}
}
\value{
a data frame containing the calculated features for all possible
protein pairs
}
\description{
Calculate the six features that are used to discriminate interacting and
non-interacting protein pairs based on co-elution profiles in PrInCE, 
namely: raw Pearson R value, cleaned Pearson R value, raw Pearson P-value,
Euclidean distance, co-peak, and co-apex. Optionally, one or more of these
can be disabled.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_ensemble.R
\name{predict_ensemble}
\alias{predict_ensemble}
\title{Predict interactions using an ensemble of classifiers}
\usage{
predict_ensemble(
  dat,
  labels,
  classifier = c("NB", "SVM", "RF", "LR"),
  models = 1,
  cv_folds = 10,
  trees = 500,
  node_columns = c(1, 2)
)
}
\arguments{
\item{dat}{a data frame containing interacting gene/protein pairs in the
first two columns, and the features to use for classification in the 
remaining columns}

\item{labels}{labels for each interaction in \code{dat}: 0 for negatives,
1 for positives, and NA for interactions outside the reference set}

\item{classifier}{the type of classifier to use; one of \code{"NB"} 
(naive Bayes), \code{"SVM"} (support vector machine), \code{"RF"}
(random forest), or \code{"LR"} (logistic regression)}

\item{models}{the number of classifiers to train}

\item{cv_folds}{the number of folds to split the reference dataset into 
when training each classifier. By default, each 
classifier uses ten-fold cross-validation, i.e., the classifier is trained
on 90\% of the dataset and used to classify the remaining 10\%}

\item{trees}{for random forest classifiers only, the number of trees to 
grow for each fold}

\item{node_columns}{a vector of length two, denoting either the indices 
(integer vector) or column names (character vector) of the columns within 
the input data frame containing the nodes participating in pairwise 
interactions; defaults to the first two columns of the data frame 
(\code{c(1, 2)})}
}
\value{
the input data frame of pairwise interactions, ranked by the 
median of classifier scores across all ensembled models
}
\description{
Use an ensemble of classifiers to predict interactions from
co-elution dataset features. The ensemble approach ensures that 
results are robust to the partitioning of the dataset into folds. For each
model, the median of classifier scores across all folds is calculated.
Then, the median of all such medians across all models is calculated.
}
\examples{
## calculate features
data(scott)
data(scott_gaussians)
subset <- scott[seq_len(500), ] ## limit to first 500 proteins
gauss <- scott_gaussians[names(scott_gaussians) \%in\% rownames(subset)]
features <- calculate_features(subset, gauss)
## make training labels
data(gold_standard)
ref <- adjacency_matrix_from_list(gold_standard)
labels <- make_labels(ref, features)
## predict interactions with naive Bayes classifier
ppi <- predict_ensemble(features, labels, classifier = "NB", 
                        cv_folds = 3, models = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_unweighted.R
\name{is_unweighted}
\alias{is_unweighted}
\title{Test whether a network is unweighted}
\usage{
is_unweighted(network)
}
\arguments{
\item{network}{the network to analyze}
}
\value{
true if the input network is a square logical or numeric matrix
}
\description{
Test whether a network is unweighted
}
\examples{
data(gold_standard)
adj <- adjacency_matrix_from_list(gold_standard)
is_unweighted(adj) ## returns TRUE

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_gaussians.R
\name{check_gaussians}
\alias{check_gaussians}
\title{Check the format of a list of Gaussians}
\usage{
check_gaussians(
  gaussians,
  proteins = NULL,
  replicate_idx = NULL,
  n_error = 3,
  pct_warning = 0.1
)
}
\arguments{
\item{gaussians}{the list of Gaussians}

\item{proteins}{the complete set of input proteins}

\item{replicate_idx}{the replicate being analyzed, if input proteins are 
provided; used to throw more informative error messages}

\item{n_error}{minimum number of proteins that can have fitted Gaussians
without throwing an error}

\item{pct_warning}{minimum fraction of proteins that can have fitted
Gaussians without giving a warning}
}
\value{
\code{TRUE} if all conditions are met, but throws an error if any 
  is not
}
\description{
Test whether an input list of Gaussians conforms to the format expected
by PrInCE: that is, a named list with five fields for each entry, i.e., the
number of Gaussians in the mixture model, the {\eqn{r^2}} value, the 
number of iterations used by \code{\link[stats]{nls}}, the coefficients of
each model, and the fitted curve.
}
\details{
Optionally, some extra checks will be 
done on the fraction of proteins in the complete dataset for which a 
Gaussian mixture model could be fit, if provided. In particular, the function
will throw an error if fewer than \code{n_error} proteins have a fitted 
Gaussian, and emit a warning if fewer than \code{pct_warning} do.
}
\examples{
data(scott_gaussians)
check_gaussians(scott_gaussians)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_autocorrelation.R
\name{calculate_autocorrelation}
\alias{calculate_autocorrelation}
\title{Calculate the autocorrelation for each protein between a pair of co-elution 
experiments.}
\usage{
calculate_autocorrelation(
  profile1,
  profile2,
  cor_method = c("pearson", "spearman", "kendall"),
  min_replicates = 1,
  min_fractions = 1,
  min_pairs = 0
)
}
\arguments{
\item{profile1}{a numeric matrix or data frame with proteins in rows and 
fractions in columns, or a \code{\linkS4class{MSnSet}} object, representing
the first co-elution condition}

\item{profile2}{a numeric matrix or data frame with proteins in rows and 
fractions in columns, or a \code{\linkS4class{MSnSet}} object, representing
the second co-elution condition}

\item{cor_method}{the correlation method to use; one of \code{"pearson"},
\code{"spearman"}, or \code{"kendall"}).}

\item{min_fractions}{filter proteins not quantified in at least this many
fractions}

\item{min_pairs}{remove correlations between protein pairs not co-occuring in
at least this many fractions from the autocorrelation calculation}
}
\value{
a named vector of autocorrelation scores for all proteins found in
both matrices.
}
\description{
For a given protein, the correlation coefficient to all other proteins 
in the first condition is calculated, yielding a vector of correlation 
coefficients. The same procedure is repeated for the second condition,
and the two vectors of correlation coefficients are themselves correlated,
yielding a metric whereby higher values reflect proteins with unchanging
interaction profiles between conditions, while lower values reflect proteins 
with substantially changing interaction profiles.
}
\details{
Note that all of zero, \code{NA}, \code{NaN}, and infinite values are
all treated equivalently as missing values when applying the 
\code{min_fractions} and \code{min_pairs} filters, but different handling of 
missing values will produce different autocorrelation scores.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_interactions.R
\name{predict_interactions}
\alias{predict_interactions}
\title{Predict interactions given a set of features and examples}
\usage{
predict_interactions(
  features,
  gold_standard,
  classifier = c("NB", "SVM", "RF", "LR", "ensemble"),
  verbose = FALSE,
  models = 10,
  cv_folds = 10,
  trees = 500
)
}
\arguments{
\item{features}{a data frame with proteins in the first two columns, and 
features to be passed to the classifier in the remaining columns}

\item{gold_standard}{an adjacency matrix of "gold standard" interactions
used to train the classifier}

\item{classifier}{the type of classifier to use: one of \code{"NB"} (naive
Bayes), \code{"SVM"} (support vector machine), \code{"RF"} (random forest),
\code{"LR"} (logistic regression), or \code{"ensemble"} (an ensemble of
all four)}

\item{verbose}{if \code{TRUE}, print a series of messages about the stage
of the analysis}

\item{models}{the number of classifiers to train and average across, each
with a different k-fold cross-validation split}

\item{cv_folds}{the number of folds to use for k-fold cross-validation}

\item{trees}{for random forests only, the number of trees in the forest}
}
\value{
a ranked data frame of pairwise interactions, with the 
classifier score, label, and cumulative precision for each interaction
}
\description{
Discriminate interacting from non-interacting protein pairs by training a
machine learning model on a set of labelled examples, given a set of 
features derived from a co-elution profile matrix (see 
\code{\link[PrInCE]{calculate_features}}.
}
\details{
PrInCE implements four different classifiers (naive Bayes, support vector
machine, random forest, and logistic regression). Naive Bayes is used as a
default. The classifiers are trained
on the gold standards using a ten-fold cross-validation procedure, training
on 90% of the data and predicting on the remaining 10%. For protein pairs
that are part of the training data, the held-out split is used to assign
a classifier score, whereas for the remaining protein pairs, the median of
all ten folds is used. Furthermore, to ensure the results are not sensitive
to the precise classifier split used, an ensemble of multiple classifiers
(ten, by default) is trained, and the classifier score is subsequently
averaged across classifiers. 

PrInCE can also ensemble across multiple different types of classifiers, 
by supplying the \code{"ensemble"} option to the \code{classifier} argument.
}
\examples{
## calculate features
data(scott)
data(scott_gaussians)
subset <- scott[seq_len(500), ] ## limit to first 500 proteins
gauss <- scott_gaussians[names(scott_gaussians) \%in\% rownames(subset)]
features <- calculate_features(subset, gauss)
## load training data
data(gold_standard)
ref <- adjacency_matrix_from_list(gold_standard)
## predict interactions
ppi <- predict_interactions(features, ref, cv_folds = 3, models = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match_matrix_dimensions.R
\name{match_matrix_dimensions}
\alias{match_matrix_dimensions}
\title{Match the dimensions of a query matrix to a profile matrix}
\usage{
match_matrix_dimensions(query, profile_matrix)
}
\arguments{
\item{query}{a square matrix containing features for pairs of proteins}

\item{profile_matrix}{the profile matrix for which interactions are being 
predicted}
}
\value{
a square matrix with the same row and column names as the input 
profile matrix, for use in interaction prediction
}
\description{
Match the row and column names of a square feature matrix to the row names
of a profile matrix, adding rows/columns containing \code{NA}s when 
proteins in the profile matrix are missing from the feature matrix.
}
\examples{
data(gold_standard)
subset <- adjacency_matrix_from_list(gold_standard[seq(1, 200)])
target <- adjacency_matrix_from_list(gold_standard)
matched <- match_matrix_dimensions(subset, target)
dim(subset)
dim(target)
dim(matched)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_weighted.R
\name{is_weighted}
\alias{is_weighted}
\title{Test whether a network is weighted}
\usage{
is_weighted(network)
}
\arguments{
\item{network}{the network to analyze}
}
\value{
true if the input network is a square numeric matrix with more than
  two values
}
\description{
Test whether a network is weighted
}
\examples{
data(gold_standard)
adj <- adjacency_matrix_from_list(gold_standard)
is_weighted(adj) ## returns FALSE

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_profiles.R
\name{filter_profiles}
\alias{filter_profiles}
\title{Filter a co-elution profile matrix}
\usage{
filter_profiles(profile_matrix, min_points = 1, min_consecutive = 5)
}
\arguments{
\item{profile_matrix}{a numeric matrix of co-elution profiles, with proteins
in rows, or a \code{\linkS4class{MSnSet}} object}

\item{min_points}{filter profiles without at least this many total, 
non-missing points}

\item{min_consecutive}{filter profiles without at least this many 
consecutive, non-missing points}
}
\value{
the filtered profile matrix
}
\description{
Filter a matrix of co-elution/co-fractionation profiles by removing
profiles without a certain number of non-mising or consecutive points.
}
\examples{
data(scott)
nrow(scott)
filtered <- filter_profiles(scott)
nrow(scott)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_precision.R
\name{calculate_precision}
\alias{calculate_precision}
\title{Calculate precision at each point in a sequence}
\usage{
calculate_precision(labels)
}
\arguments{
\item{labels}{a vector of zeroes (FPs) and ones (TPs)}
}
\value{
a vector of the same length giving the precision at each point in the
input vector
}
\description{
Calculate the precision of a list of interactions at each point in the list,
given a set of labels.
}
\examples{
## calculate features
data(scott)
data(scott_gaussians)
subset <- scott[seq_len(500), ] ## limit to first 500 proteins
gauss <- scott_gaussians[names(scott_gaussians) \%in\% rownames(subset)]
features <- calculate_features(subset, gauss)
## make training labels
data(gold_standard)
ref <- adjacency_matrix_from_list(gold_standard)
labels <- make_labels(ref, features)
## predict interactions with naive Bayes classifier
ppi <- predict_ensemble(features, labels, classifier = "NB", cv_folds = 3, 
                        models = 1)
## tag precision of each interaction
ppi$precision <- calculate_precision(ppi$label)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_neighbors.R
\name{impute_neighbors}
\alias{impute_neighbors}
\title{Impute single missing values}
\usage{
impute_neighbors(chromatogram)
}
\arguments{
\item{chromatogram}{a numeric vector corresponding to the chromatogram trace}
}
\value{
the imputed chromatogram
}
\description{
Impute single missing values within a chromatogram profile as the average
of their neighbors.
}
\examples{
data(scott)
chrom <- scott[16, ]
imputed <- impute_neighbors(chrom)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kristensen.R
\docType{data}
\name{kristensen}
\alias{kristensen}
\title{Interactome of HeLa cells}
\format{
a data frame with 1875 rows and 48 columns, with proteins in rows and
  SEC fractions in columns
}
\source{
\url{https://www.nature.com/articles/nmeth.2131}
}
\usage{
data(kristensen)
}
\description{
Co-elution profiles derived from size exclusion chromatography (SEC) of
HeLa cell lysates.
}
\details{
Protein quantitation was accomplished by SILAC (stable isotopic labelling
by amino acids in cell culture), and is ratiometric, i.e., it reflects 
the ratio between the intensity of the heavy isotope and the light isotope 
("H/L"). The dataset was initially described in Kristensen et al., 
\emph{Nat. Methods} 2012. The medium isotope channel from replicate
1 (Supplementary Table 1a in the online supplementary information) is 
included in the \code{PrInCE} package. The R script used to generate
this matrix from the supplementary materials of the paper is provided in the
\code{data-raw} directory of the package source code.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threshold_precision.R
\name{threshold_precision}
\alias{threshold_precision}
\title{Threshold interactions at a given precision cutoff}
\usage{
threshold_precision(interactions, threshold)
}
\arguments{
\item{interactions}{the ranked list of interactions output by
\code{\link[PrInCE]{predict_interactions}}, including a \code{precision}
column}

\item{threshold}{the minimum precision of the unweighted interaction
network to return}
}
\value{
the subset of the original ranked list at the given precision
}
\description{
Threshold interactions at a given precision cutoff
}
\examples{
data(scott)
data(scott_gaussians)
data(gold_standard)
# analyze only the first 100 profiles
subset <- scott[seq_len(500), ]
gauss <- scott_gaussians[names(scott_gaussians) \%in\% rownames(subset)]
ppi <- PrInCE(subset, gold_standard,
  gaussians = gauss, models = 1,
  cv_folds = 3
)
network <- threshold_precision(ppi, threshold = 0.5)
nrow(network)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjacency_matrix_from_data_frame.R
\name{adjacency_matrix_from_data_frame}
\alias{adjacency_matrix_from_data_frame}
\title{Create an adjacency matrix from a data frame}
\usage{
adjacency_matrix_from_data_frame(dat, symmetric = TRUE, node_columns = c(1, 2))
}
\arguments{
\item{dat}{a data frame containing pairwise interactions}

\item{symmetric}{if true, interactions in both directions will be 
added to the adjacency matrix}

\item{node_columns}{a vector of length two, denoting either the indices 
(integer vector) or column names (character vector) of the columns within 
the data frame containing the nodes participating in pairwise interactions;
defaults to the first two columns of the data frame (\code{c(1, 2)})}
}
\value{
an adjacency matrix between all interacting proteins
}
\description{
Convert a data frame containing pairwise interactions into an
adjacency matrix. The resulting square adjacency matrix contains ones for 
proteins that are found in interactions and zeroes otherwise.
}
\examples{
ppi <- data.frame(protein_A = paste0("protein", seq_len(10)),
                  protein_B = paste0("protein", c(rep(3, 2), rep(5, 5), 
                                     rep(7, 3))))
adj <- adjacency_matrix_from_data_frame(ppi)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/choose_gaussians.R
\name{choose_gaussians}
\alias{choose_gaussians}
\title{Fit a Gaussian mixture model to a co-elution profile}
\usage{
choose_gaussians(
  chromatogram,
  points = NULL,
  max_gaussians = 5,
  criterion = c("AICc", "AIC", "BIC"),
  max_iterations = 10,
  min_R_squared = 0.5,
  method = c("guess", "random"),
  filter_gaussians_center = TRUE,
  filter_gaussians_height = 0.15,
  filter_gaussians_variance_min = 0.1,
  filter_gaussians_variance_max = 50
)
}
\arguments{
\item{chromatogram}{a numeric vector corresponding to the chromatogram trace}

\item{points}{optional, the number of non-NA points in the raw data}

\item{max_gaussians}{the maximum number of Gaussians to fit; defaults to 5.
Note that Gaussian mixtures with more parameters than observed (i.e., 
non-zero or NA) points will not be fit.}

\item{criterion}{the criterion to use for model selection;
one of "AICc" (corrected AIC, and default), "AIC", or "BIC"}

\item{max_iterations}{the number of times to try fitting the curve with
different initial conditions; defaults to 10}

\item{min_R_squared}{the minimum R-squared value to accept when fitting the
curve with different initial conditions; defaults to 0.5}

\item{method}{the method used to select the initial conditions for
nonlinear least squares optimization (one of "guess" or "random"); 
see \code{\link{make_initial_conditions}} for details}

\item{filter_gaussians_center}{true or false: filter Gaussians whose centres
fall outside the bounds of the chromatogram}

\item{filter_gaussians_height}{Gaussians whose heights are below this 
fraction of the chromatogram height will be filtered. Setting this value to
zero disables height-based filtering of fit Gaussians}

\item{filter_gaussians_variance_min}{Gaussians whose variance is below this 
threshold will be filtered. Setting this value to zero disables filtering.}

\item{filter_gaussians_variance_max}{Gaussians whose variance is above this 
threshold will be filtered. Setting this value to zero disables filtering.}
}
\value{
a list with five entries: the number of Gaussians used to fit
the curve; the R^2 of the fit; the number of iterations used to 
fit the curve with different initial conditions; the coefficients of the 
fit model; and the curve predicted by the fit model.
}
\description{
Fit mixtures of one or more Gaussians to the curve formed by a chromatogram
profile, and choose the best fitting model using an information criterion
of choice.
}
\examples{
data(scott)
chrom <- clean_profile(scott[1, ])
gauss <- choose_gaussians(chrom, max_gaussians = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_complexes.R
\name{detect_complexes}
\alias{detect_complexes}
\title{Detect significantly interacting complexes in a chromatogram matrix}
\usage{
detect_complexes(
  profile_matrix,
  complexes,
  method = c("pearson", "euclidean"),
  min_pairs = 10,
  bootstraps = 100,
  progress = TRUE
)
}
\arguments{
\item{profile_matrix}{a matrix of chromatograms, with proteins in the rows
and fractions in the columns, or a \code{\linkS4class{MSnSet}} object}

\item{complexes}{a named list of protein complexes, where the name is the
complex name and the entries are proteins within that complex}

\item{method}{method to use to calculate edge weights;
one of \code{pearson} or \code{euclidean}}

\item{min_pairs}{the minimum number of pairwise observations to count a 
correlation or distance towards the z score}

\item{bootstraps}{number of bootstraps to execute to estimate z scores}

\item{progress}{whether to show the progress of the function}
}
\value{
a named vector of z scores for each complex in the input list
}
\description{
Use a permutation testing approach to identify complexes that show a 
significant tendency to interact, relative to random sets of complexes
of equivalent size. The function begins by calculating the Pearson 
correlation or Euclidean distance between all proteins in the matrix, and
}
\examples{
data(scott)
data(gold_standard)
complexes <- gold_standard[lengths(gold_standard) >= 3]
z_scores <- detect_complexes(t(scott), complexes)
length(na.omit(z_scores)) ## number of complexes that could be tested
z_scores[which.max(z_scores)] ## most significant complex

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gold_standard.R
\docType{data}
\name{gold_standard}
\alias{gold_standard}
\title{Reference set of human protein complexes}
\format{
a list containing 467 entries (character vectors)
}
\source{
\url{https://www.ebi.ac.uk/complexportal/complex/organisms}
}
\usage{
data(gold_standard)
}
\description{
A reference set of 467 experimentally confirmed human protein complexes,
derived from the EBI Complex Portal database.
}
\details{
467 protein complexes, ranging in size from 2 to 44 proteins and involving
877 proteins in total, to provide a reference set of true positive and true 
negative interactions (intra- and inter-complex interactions, respectively)
for demonstration in PrInCE analysis of a co-elution dataset.
Other "gold standards" are possible in practice, most notably the CORUM
database; however, the Complex Portal reference set is included in this
package due to its CC-BY license.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scott.R
\docType{data}
\name{scott}
\alias{scott}
\title{Cytoplasmic interactome of Jurkat T cells during apoptosis}
\format{
a data frame with 1560 rows and 55 columns, with proteins in rows and
  SEC fractions in columns
}
\source{
\url{http://msb.embopress.org/content/13/1/906}
}
\usage{
data(scott)
}
\description{
Co-elution profiles derived from size exclusion chromatography (SEC) of
cytoplasmic fractions from Jurkat T cells, 4 hours following Fas stimulation.
}
\details{
Protein quantitation was accomplished by SILAC (stable isotopic labelling
by amino acids in cell culture), and is ratiometric, i.e., it reflects 
the ratio between the intensity of the heavy isotope and the light isotope 
("H/L"). The dataset was initially described in Scott et al., 
\emph{Mol. Syst. Biol.} 2017. The heavy isotope channel from replicate
1 is included in the \code{PrInCE} package. The R script used to generate
this matrix from the supplementary materials of the paper is provided in the
\code{data-raw} directory of the package source code.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_labels.R
\name{make_labels}
\alias{make_labels}
\title{Make labels for a classifier based on a gold standard}
\usage{
make_labels(gold_standard, dat, node_columns = c(1, 2), protein_groups = NULL)
}
\arguments{
\item{gold_standard}{an adjacency matrix of gold-standard interactions}

\item{dat}{a data frame with interacting proteins in the first two
columns}

\item{node_columns}{a vector of length two, denoting either the indices 
(integer vector) or column names (character vector) of the columns within 
the data frame containing the nodes participating in pairwise interactions;
defaults to the first two columns of the data frame (\code{c(1, 2)})}

\item{protein_groups}{optionally, specify a list linking each protein in the 
first two columns of the input data frame to a protein group}
}
\value{
a vector of the same length as the input dataset, containing 
\code{NA}s for protein pairs not in the gold standard and ones or zeroes
based on the content of the adjacency matrix
}
\description{
Create labels for a classifier for protein pairs in the same order as 
in a dataset that will be used as input to a classifier, in a 
memory-friendly way.
}
\examples{
data(gold_standard)
adj <- adjacency_matrix_from_list(gold_standard)
proteins <- unique(unlist(gold_standard))
dat <- data.frame(protein_A = sample(proteins, 10), 
                  protein_B = sample(proteins, 10))
labels <- make_labels(adj, dat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_feature_from_data_frame.R
\name{make_feature_from_data_frame}
\alias{make_feature_from_data_frame}
\title{Create a feature vector for a classifier from a data frame}
\usage{
make_feature_from_data_frame(
  dat,
  target,
  dat_node_cols = c(1, 2),
  target_node_cols = c(1, 2),
  feature_col = 3,
  default_value = NA
)
}
\arguments{
\item{dat}{a data frame containing pairwise interactions and a feature to be
converted to a vector in a third column}

\item{target}{the data frame of features that will be provided as input to a 
classifier}

\item{dat_node_cols}{a vector of length two, denoting either the indices 
(integer vector) or column names (character vector) of the columns within 
the feature data frame; defaults to the first two columns of the data 
frame (\code{c(1, 2)})}

\item{target_node_cols}{a vector of length two, denoting either the indices 
(integer vector) or column names (character vector) of the columns within 
the target data frame; defaults to the first two columns of the data 
frame (\code{c(1, 2)})}

\item{feature_col}{the name or index of the column in the first data frame 
that contains a feature for each interaction}

\item{default_value}{the default value for protein pairs that are not in the 
first data frame (set, by default, to \code{NA})}
}
\value{
a vector matching the dimensions and order of the feature data frame,
to use as input for a classifier in interaction prediction
}
\description{
Convert a data frame containing pairwise interactions, and  
a score or other data associated with each interaction, into a feature vector
that matches the dimensions of a data frame used as input to a classifier, 
such as a naive Bayes, random forests, or support vector machine classifier.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_feature_from_expression.R
\name{make_feature_from_expression}
\alias{make_feature_from_expression}
\title{Create a feature vector from expression data}
\usage{
make_feature_from_expression(expr, dat, node_columns = c(1, 2), ...)
}
\arguments{
\item{expr}{a matrix containing gene or protein expression data, with 
genes/proteins in columns and samples in rows}

\item{dat}{the data frame of features to be used by the classifier, 
with protein pairs in the columns specified by the \code{node_columns}
argument}

\item{node_columns}{a vector of length two, denoting either the indices 
(integer vector) or column names (character vector) of the columns within 
the data frame containing the nodes participating in pairwise interactions;
defaults to the first two columns of the data frame (\code{c(1, 2)})}

\item{...}{arguments passed to \code{cor}}
}
\value{
a vector matching the dimensions and order of the feature data frame,
to use as input for a classifier in interaction prediction
}
\description{
Convert a gene or protein expression matrix into a feature vector
that matches the dimensions of a data frame used as input to a classifier, 
such as a naive Bayes, random forests, or support vector machine classifier,
by calculating the correlation between each pair of genes or proteins.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concatenate_features.R
\name{concatenate_features}
\alias{concatenate_features}
\title{Combine features across multiple replicates}
\usage{
concatenate_features(feature_list)
}
\arguments{
\item{feature_list}{a list of feature data frames, as produced by 
\code{\link[PrInCE]{calculate_features}}, with proteins in the first two 
columns}
}
\value{
a data frame containing features for all protein pairs across all 
  replicates
}
\description{
Concatenate features extracted from multiple replicates to a single data 
frame that will be used as input to a classifier. Doing so allows the 
classifier to naturally weight evidence for an interaction between each
protein pair from each feature in each replicate in proportion to its
discriminatory power on known examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/co_apex.R
\name{co_apex}
\alias{co_apex}
\title{Calculate the co-apex score for every protein pair}
\usage{
co_apex(gaussians, proteins = NULL)
}
\arguments{
\item{gaussians}{a list of Gaussian mixture models fit to the profile matrix
by \code{link{build_gaussians}}}

\item{proteins}{all proteins being scored, optionally including those 
without Gaussian fits}
}
\value{
a matrix of co-apex scores
}
\description{
Calculate the co-apex score for every pair of proteins. This is defined as 
the minimum Euclidean distance between any two Gaussians fit to each 
profile.
}
\examples{
data(scott_gaussians)
gauss <- scott_gaussians[seq_len(25)]
CA <- co_apex(gauss)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_curve.R
\name{fit_curve}
\alias{fit_curve}
\title{Output the fit curve for a given mixture of Gaussians}
\usage{
fit_curve(coef, indices)
}
\arguments{
\item{coef}{numeric vector of coefficients for a Gaussian mixture model fit 
by \code{\link{fit_gaussians}}. This function assumes that the heights of the
Gaussians are specified by coefficients beginning with "A" 
("A1", "A2", "A3", etc.), centres are specified by coefficients beginning
with "mu", and standard deviations are specified by coefficients beginning
with "sigma".}

\item{indices}{the indices, or x-values, to predict a fitted curve for 
(for example, the fractions in a given chromatogram)}
}
\value{
the fitted curve
}
\description{
For a Gaussian mixture model fit to a curve by \code{\link{fit_gaussians}},
output the fit curve using the coefficients rather than the \code{nls} 
object. This allows individual Gaussians to be removed from the fit model:
for example, if their height is below a certain threshold, or their 
centres are outside the bounds of the chromatogram.
}
\examples{
data(scott)
chrom <- clean_profile(scott[1, ])
fit <- fit_gaussians(chrom, n_gaussians = 1)
curve <- fit_curve(fit$coefs, seq_along(chrom))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_profile.R
\name{clean_profile}
\alias{clean_profile}
\title{Preprocess a co-elution profile}
\usage{
clean_profile(
  chromatogram,
  impute_NA = TRUE,
  smooth = TRUE,
  smooth_width = 4,
  noise_floor = 0.001
)
}
\arguments{
\item{chromatogram}{a numeric vector corresponding to the chromatogram trace}

\item{impute_NA}{if true, impute single missing values with the average of
neighboring values}

\item{smooth}{if true, smooth the chromatogram with a moving average filter}

\item{smooth_width}{width of the moving average filter, in fractions}

\item{noise_floor}{mean value of the near-zero noise to add}
}
\value{
a cleaned profile
}
\description{
Clean a co-elution/co-fractionation profile by (1) imputing single missing
values with the average of neighboring values, (2) replacing missing values
with random, near-zero noise, and (3) smoothing with a moving average
filter.
}
\examples{
data(scott)
chrom <- scott[16, ]
cleaned <- clean_profile(chrom)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scott_gaussians.R
\docType{data}
\name{scott_gaussians}
\alias{scott_gaussians}
\title{Fitted Gaussian mixture models for the \code{scott} dataset}
\format{
a named list with 970 entries; names are proteins, and list items 
  conain information about fitted Gaussians in the format that PrInCE expects
}
\usage{
data(scott_gaussians)
}
\description{
The \code{\link[PrInCE]{scott}} dataset consists of protein
co-migration profiles derived from size exclusion chromatography (SEC) of
cytoplasmic fractions from Jurkat T cells, 4 hours following Fas stimulation.
The \code{scott_gaussians} object contains Gaussian mixture models fit by
the function \code{\link[PrInCE]{build_gaussians}}; this is bundled with the
R package in order to expedite the demonstration code, as the process of 
Gaussian fitting is one of the more time-consuming aspects of the package.
}
\details{
As with the \code{\link[PrInCE]{scott}} dataset, the code used to generate
this data object is provided in the \code{data-raw} directory of the package
source.
}
\keyword{datasets}
